program ex1_apply_q_bd
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutines APPLY_Q_BD and APPLY_P_BD
!   in module SVD_Procedures.
!                                                                              
!                                                     
! Further Details
! ===============
!
!    The program also shows the use of subroutines BD_CMP, BD_SVD and BD_INVITER
!    in module SVD_Procedures.
!
! LATEST REVISION : 10/06/2017
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : lgl, i4b, stnd, true, false, zero, bd_inviter,     &
                         bd_cmp, bd_svd, apply_q_bd, apply_p_bd, norm, c50
#ifdef _MATMUL
    use Statpack, only : matmul=>matmul2
#endif
#ifdef _TRANSPOSE
    use Statpack, only : transpose=>transpose2
#endif
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=1000, m=500, mn=min(m,n), nsing=20
!   
    real(stnd), parameter  :: fudge=c50
!
    character(len=*), parameter :: name_proc='Example 1 of apply_q_bd'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                     :: err, eps
    real(stnd), dimension(n,m)     :: a, a2
    real(stnd), dimension(n,nsing) :: leftvec
    real(stnd), dimension(m,nsing) :: rightvec
    real(stnd), dimension(mn)      :: s, d, e, e2, tauq, taup
!
    integer(i4b) :: maxiter=2
!
    logical(lgl) :: failure, bd_is_upper
!   
    character    :: sort='d'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = fudge*epsilon( err )
!
!   GENERATE A RANDOM DATA MATRIX a.
!
    call random_number( a )
!
!   SAVE RANDOM DATA MATRIX a .
!
    a2(:n,:m) = a(:n,:m)
!
!   REDUCE a TO LOWER OR UPPER BIDIAGONAL FORM.
!
    call bd_cmp( a, s, e2, tauq, taup )
!
!   THE DIAGONAL ELEMENTS ARE STORED IN s .
!   THE OFF-DIAGONAL ELEMENTS ARE STORED IN e2 .
!
!   SAVE BIDIAGONAL FORM OF a .
!
     e(:mn) = e2(:mn)
     d(:mn) = s(:mn)
!
     bd_is_upper = n>=m
!
!    COMPUTE SINGULAR VALUES OF BIDIAGONAL FORM OF a .
!
     call bd_svd( bd_is_upper, s, e2, failure, sort=sort  )
!
     if ( .not. failure ) then
!
!       COMPUTE THE FIRST nsing SINGULAR VECTORS OF a BY maxiter
!       INVERSE ITERATIONS ON THE INTERMEDIATE BIDIAGONAL MATRIX
!       d_e AND BACK-TRANSFORMATION.
!
        call bd_inviter( bd_is_upper, d, e, s(:nsing), leftvec(:mn,:nsing), rightvec(:mn,:nsing),  &
                         failure, maxiter=maxiter )
!
!       COMPUTE SINGULAR VECTORS OF a BY BACK-TRANSFORMATION.
!
        if ( bd_is_upper ) then
            leftvec(mn+1_i4b:n,:nsing) = zero
        else
            rightvec(mn+1_i4b:m,:nsing) = zero
        end if
!
!       GENERATE LEFT SINGULAR VECTORS OF a IN leftvec BY BACK-TRANSFORMATION.
!
        call apply_q_bd( a, tauq, leftvec,  left=true, trans=false )
!
!       GENERATE RIGHT SINGULAR VECTORS OF a IN rightvec BY BACK-TRANSFORMATION.
!
        call apply_p_bd( a, taup, rightvec, left=true, trans=false )
!
!       CHECK FOR SMALL RESIDUALS OF THE EXPRESSION a*rightvec - leftvec*s(:nsing),
!       WHERE leftvec AND rightvec ARE THE LEFT AND RIGHT SINGULAR VECTORS.
!
        err =  norm( matmul(a2,rightvec) - leftvec*spread(s(:nsing),dim=1,ncopies=n) )/( sum( abs(s(:mn)) )*real(mn,stnd) )
!
    end if
!
    if ( err<=eps .and. .not.failure ) then
        write (prtunit,*) name_proc//' is correct'
    else
        write (prtunit,*) name_proc//' is incorrect'
    end if
!
!
! END OF PROGRAM ex1_apply_q_bd
! =============================
!
end program ex1_apply_q_bd
