program ex1_comp_cor_miss2
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutine COMP_COR_MISS2
!   in module Mul_Stat_Procedures .
!                                                                              
! LATEST REVISION : 26/06/2006
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, true, comp_cor_miss2
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT.
!
    integer(i4b), parameter :: prtunit=6, n=26, m=20, p=50
!
! miss IS THE MISSING INDICATOR.
!
    real(stnd), parameter :: miss=-999.99_stnd
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                    :: err_xstat, err_ystat, err_cor, eps
    real(stnd), dimension(n,m)    :: xycor1, xycor2, xyn
    real(stnd), dimension(n,m,2)  :: xstat1, xstat2, ystat1, ystat2
    real(stnd), dimension(n,m,p)  :: x
    real(stnd), dimension(p)      :: y
!
    integer(i4b) :: i
!
    logical(lgl) :: first, last
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 1 of comp_cor_miss2'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
    eps = sqrt( epsilon(eps) )
!
!   GENERATE A RANDOM TRIDIMENSIONAL OBSERVATION ARRAY x WITH MISSING VALUES.
!
    call random_number( x(:n,:m,:p) )
    where ( x(:n,:m,:p)<=0.05_stnd ) x(:n,:m,:p) = miss
!
!   GENERATE A RANDOM OBSERVATION VECTOR y WITH MISSING VALUES.
!
    call random_number( y(:p) )
    where ( y(:p)<=0.05_stnd ) y(:p) = miss
!
!   COMPUTE THE CORRELATIONS BETWEEN x AND y
!   FOR THE p OBSERVATIONS .
!
    first = true
    last  = true
    call comp_cor_miss2( x(:n,:m,:p), y(:p), first, last, xstat1(:n,:m,:2),               &
                        ystat1(:n,:m,:2), xycor1(:n,:m), xyn(:n,:m), xymiss=miss )
!
!   ON EXIT OF COMP_COR_MISS2 WHEN last=true :
!
!      xstat1(i,j,1) CONTAINS THE MEAN VALUES OF THE ARRAY SECTION x(i,j,:p),
!                    COMPUTED FROM VALID PAIRS OF OBSERVATIONS FOR  x(i,j,:p) AND y(:p).
!
!      xstat1(i,j,2) CONTAINS THE VARIANCES OF THE ARRAY SECTION x(i,j,:p),
!                    COMPUTED FROM VALID PAIRS OF OBSERVATIONS FOR  x(i,j,:p) AND y(:p).
!
!      ystat1(i,j,1) CONTAINS THE MEAN VALUE OF THE DATA VECTOR y(:p),
!                    COMPUTED FROM VALID PAIRS OF OBSERVATIONS FOR  x(i,j,:p) AND y(:p).
!
!      ystat1(i,j,2) CONTAINS THE VARIANCE OF THE DATA VECTOR y(:p),
!                    COMPUTED FROM VALID PAIRS OF OBSERVATIONS FOR  x(i,j,:p) AND y(:p).
!
!      xycor1(i,j)   CONTAINS THE CORRELATION COEFFICIENT BETWEEN x(i,j,:p) AND y(:p)
!                    COMPUTED WITH THE ABOVE UNIVARIATE STATISTICS.
!
!      xyn(i,j)      CONTAINS THE INCIDENCE VALUE BETWEEN x(i,j,:p) AND y(:p).
!                    xycor1(i,j) INDICATES THE NUMBER OF VALID PAIRS OF OBSERVATIONS
!                    WHICH WHERE USED IN THE CALCULATION OF ALL THE ABOVE STATISTICS .
!
!
!   COMPUTE CORRELATIONS BETWEEN x AND y,
!   ITERATIVELY  FOR THE p OBSERVATIONS .
!
    do i = 1, p
!
        first = i==1
        last  = i==p
!
        call comp_cor_miss2( x(:n,:m,i:i), y(i:i), first, last, xstat2(:n,:m,:2),        &
                            ystat2(:n,:m,:2), xycor2(:n,:m), xyn(:n,:m), xymiss=miss )
    end do
!
!   CHECK THAT THE TWO SETS OF STATISTICS AGREE.
!
    err_xstat = maxval( abs( ( xstat1(:n,:m,:2)-xstat2(:n,:m,:2))/xstat1(:n,:m,:2) ) )
    err_ystat = maxval( abs( ( ystat1(:n,:m,:2)-ystat2(:n,:m,:2))/ystat1(:n,:m,:2) ) )
    err_cor   = maxval( abs( xycor1(:n,:m)-xycor2(:n,:m) ) )
!
    if ( max(err_xstat, err_ystat, err_cor )<=eps ) then
        write (prtunit,*) 'Example 1 of COMP_COR_MISS2 is correct'
    else
        write (prtunit,*) 'Example 1 of COMP_COR_MISS2 is incorrect'
    end if
!
!
! END OF PROGRAM ex1_comp_cor_miss2
! =================================
!
end program ex1_comp_cor_miss2
