program ex1_comp_cormat
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutine COMP_CORMAT
!   in module Mul_Stat_Procedures .
!                                                                              
! LATEST REVISION : 26/06/2006
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, true, false, comp_cormat
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, m=20, p=500
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                         :: err_mean, err_std, err_cor, eps, xn
    real(stnd), dimension(m,m)         :: cor1, cor2
    real(stnd), dimension(m,p)         :: x
    real(stnd), dimension(m)           :: mean1, mean2, std1, std2
!
    integer(i4b) :: i
!
    logical(lgl) :: first, last, cov, fill
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 1 of comp_cormat'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
    eps = sqrt( epsilon(eps) )
!
!   GENERATE A RANDOM OBSERVATION ARRAY x .
!
    call random_number( x )
!
    cov  = false
    fill = true
!
!   COMPUTE THE MEANS, STANDARD-DEVIATIONS AND CORRELATION MATRIX
!   OF x FOR THE p OBSERVATIONS .
!
    first = true
    last  = true
    call comp_cormat( x(:m,:p), first, last, mean1(:m), cor1(:m,:m), xn,    &
                      xstd=std1(:m), cov=cov, fill=fill )
!
!   ON EXIT, WHEN last=true :
!
!     mean1(:m) CONTAINS THE VARIABLE MEANS COMPUTED FROM ALL OBSERVATIONS 
!     IN THE DATA MATRIX x.
!
!     cor1(:m,:m) CONTAINS THE UPPER TRIANGLE OF THE SYMETRIC CORRELATION
!     OR VARIANCE-COVARIANCE MATRIX AS CONTROLLED BY THE cov ARGUMENT.
!     IF THE OPTIONAL ARGUMENT fill IS PRESENT AND EQUAL TO true,
!     THE LOWER TRIANGLE OF cor1 IS ALSO FILLED. OTHERWISE, THE LOWER TRIANGLE
!     OF cor1 IS NOT MODIFIED.
!
!     xn INDICATES THE NUMBERS OF OBSERVATIONS WHICH WERE 
!     USED IN THE CALCULATION OF cor1
!
!     IF THE OPTIONAL ARGUMENT xstd IS PRESENT, xstd CONTAINS THE VARIABLE 
!     STANDARD-DEVIATIONS.
!
!   COMPUTE THE MEANS, STANDARD-DEVIATIONS AND CORRELATION MATRIX
!   OF x, ITERATIVELY  FOR THE p OBSERVATIONS .
!
    do i = 1, p
!
        first = i==1
        last  = i==p
!
        call comp_cormat( x(:m,i:i), first, last, mean2(:m), cor2(:m,:m), xn,    &
                         xstd=std2(:m), cov=cov, fill=fill )
    end do
!
!   CHECK THAT THE TWO SETS OF STATISTICS AGREE.
!
    err_mean = maxval( abs( ( mean1-mean2)/mean1 ) )
    err_std  = maxval( abs( ( std1-std2)/std1    ) )
    err_cor  = maxval( abs( cor1-cor2 ) )
!
    if ( max(err_mean, err_std, err_cor )<=eps ) then
        write (prtunit,*) 'Example 1 of COMP_CORMAT is correct'
    else
        write (prtunit,*) 'Example 1 of COMP_CORMAT is incorrect'
    end if
!
!
! END OF PROGRAM ex1_comp_cormat
! ==============================
!
end program ex1_comp_cormat
