program ex1_comp_cormat_miss
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutine COMP_CORMAT_MISS
!   in module Mul_Stat_Procedures .
!                                                                              
! LATEST REVISION : 26/06/2006
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, true, false, comp_cormat_miss
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, m=20, p=500, n=(m*(m+1))/2
!
! miss IS THE MISSING INDICATOR.
!
    real(stnd), parameter :: miss=-999.99_stnd
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                         :: err_mean, err_std, err_cor, eps
    real(stnd), dimension(m,m)         :: cor1, cor2
    real(stnd), dimension(m,p)         :: x
    real(stnd), dimension(n,3)         :: xn
    real(stnd), dimension(m,2)         :: mean1, mean2
    real(stnd), dimension(m)           :: std1, std2
!
    integer(i4b) :: i
!
    logical(lgl) :: first, last, cov, fill
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 1 of comp_cormat_miss'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
    eps = sqrt( epsilon(eps) )
!
!   GENERATE A RANDOM OBSERVATION ARRAY x WITH MISSING VALUES.
!
    call random_number( x(:m,:p) )
    where ( x(:m,:p)<=0.05_stnd ) x(:m,:p) = miss
!
    cov  = false
    fill = true
!
!   COMPUTE THE MEANS, STANDARD-DEVIATIONS AND CORRELATION MATRIX
!   OF x FOR THE p OBSERVATIONS .
!
    first = true
    last  = true
    call comp_cormat_miss( x(:m,:p), first, last, mean1(:m,:2), cor1(:m,:m), xn(:n,:3), miss,   &
                           xstd=std1(:m), cov=cov, fill=fill )
!
!   ON EXIT, WHEN last=true :
!
!     mean1(:m,1) CONTAINS THE VARIABLE MEANS COMPUTED FROM ALL NON-MISSING OBSERVATIONS 
!     IN THE DATA MATRIX x. mean1(:m,2) IS USED AS WORKSPACE.
!
!     cor1(:m,:m) CONTAINS THE UPPER TRIANGLE OF THE SYMETRIC CORRELATION
!     OR VARIANCE-COVARIANCE MATRIX AS CONTROLLED BY THE cov ARGUMENT.
!     IF THE OPTIONAL ARGUMENT fill IS PRESENT AND EQUAL TO true,
!     THE LOWER TRIANGLE OF cor1 IS ALSO FILLED. OTHERWISE, THE LOWER TRIANGLE
!     OF cor1 IS NOT MODIFIED.
!
!     xn(:n,1) CONTAINS THE UPPER TRIANGLE OF THE MATRIX OF THE INCIDENCE VALUES
!     BETWEEN EACH PAIR OF VARIABLES, PACKED COLUMNWISE, IN A LINEAR ARRAY. 
!     xn(i + (j-1)*j/2,1) INDICATES THE NUMBERS OF NON-MISSING PAIRS WHICH WERE 
!     USED IN THE CALCULATION OF cor1(i,j) for 1<=i<=j .
!     xn(:n,2:3) IS USED AS WORKSPACE.
!
!     IF THE OPTIONAL ARGUMENT xstd IS PRESENT, xstd CONTAINS THE VARIABLE 
!     STANDARD-DEVIATIONS COMPUTED FROM ALL NON-MISSING OBSERVATIONS.
!
!   COMPUTE THE MEANS, STANDARD-DEVIATIONS AND CORRELATION MATRIX
!   OF x, ITERATIVELY  FOR THE p OBSERVATIONS .
!
    do i = 1, p
!
        first = i==1
        last  = i==p
!
        call comp_cormat_miss( x(:m,i:i), first, last, mean2(:m,:2), cor2(:m,:m), xn(:n,:3), miss,   &
                               xstd=std2(:m), cov=cov, fill=fill )
    end do
!
!   CHECK THAT THE TWO SETS OF STATISTICS AGREE.
!
    err_mean = maxval( abs( ( mean1(:m,1)-mean2(:m,1))/mean1(:m,1) ) )
    err_std  = maxval( abs( ( std1(:m)-std2(:m))/std1(:m)    ) )
    err_cor  = maxval( abs( cor1(:m,:m)-cor2(:m,:m) ) )
!
    if ( max(err_mean, err_std, err_cor )<=eps ) then
        write (prtunit,*) 'Example 1 of COMP_CORMAT_MISS is correct'
    else
        write (prtunit,*) 'Example 1 of COMP_CORMAT_MISS is incorrect'
    end if
!
!
! END OF PROGRAM ex1_comp_cormat_miss
! ===================================
!
end program ex1_comp_cormat_miss
