program ex1_comp_eof2
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutines COMP_EOF2
!   and COMP_PC_EOF in module Mul_Stat_Procedures .
!                                                                              
! LATEST REVISION : 26/06/2006
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, true, false, one, comp_eof2, comp_pc_eof
#ifdef _MATMUL
    use Statpack, only : matmul=>matmul2
#endif
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, m=20, mm=(m*(m+1))/2, p=50, neig=3
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                         :: err_pc, xn
    real(stnd), dimension(m)           :: mean, std, eigval, eigvar, singval
    real(stnd), dimension(mm)          :: corp
    real(stnd), dimension(m,p)         :: x, x_std
    real(stnd), dimension(m,neig)      :: eigvec
    real(stnd), dimension(p,neig)      :: pc
!
    integer(i4b) :: maxiter=4
!
    logical(lgl) :: first, last, failure
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 1 of comp_eof2'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   GENERATE A RANDOM OBSERVATION MATRIX x(:m,:p) WITH m VARIABLES
!   AND p OBSERVATIONS.
!
    call random_number( x )
!
!   COMPUTE neig EOFs FROM THE CORRELATION MATRIX BY INVERSE ITERATION.
!
    first = true
    last  = true
!
    call comp_eof2( x(:m,:p), first, last, eigval(:m), corp(:mm), xn, failure,   &
                    maxiter=maxiter, xmean=mean(:m), xstd=std(:m),               &
                    xeigvar=eigvar(:m), xeigvec=eigvec(:m,:neig)                 )
!
!   ON EXIT OF COMP_EOF2 WHEN last=true :
!
!      eigval(:m)       CONTAINS THE EIGENVALUES.
!
!      failure = false  INDICATES SUCCESSFUL EXIT.
!      failure = true   INDICATES THAT FEWER THAN TWO VALID OBSERVATIONS
!                       WERE PRESENT OR THAT THE OBSERVATIONS ON SOME
!                       VARIABLE WERE CONSTANT AND THE CORRELATIONS
!                       WERE REQUESTED OR THAT MAXIMUM ACCURACY WAS NOT
!                       ACHIEVED WHEN COMPUTING THE EIGENVALUES OR THAT
!                       SOME EIGENVECTORS FAILED TO CONVERGE WITH maxiter
!                       INVERSE ITERATIONS.
!
!      xn               CONTAINS THE NUMBER OF OBSERVATIONS IN THE DATA ARRAY x,
!                       xn = real(p,stnd) .
!
!      mean(:m)         CONTAINS THE MEAN VALUES OF THE m VARIABLES.
!
!      std(:m)          CONTAINS THE STANDARD-DEVIATIONS OF THE m VARIABLES.
!
!      eigvar(:m)       CONTAINS PERCENTAGES OF TOTAL VARIANCE ASSOCIATED 
!                       WITH THE EIGENVECTORS IN THE ORDER OF THE EIGENVALUES
!                       STORED IN eigval.
!
!      eigvec(:m,:neig) CONTAINS THE FIRST neig EIGENVECTORS STORED COLUMNWISE
!                       IN THE ORDER OF THE EIGENVALUES STORED IN eigval.
!
!
!   COMPUTE THE PRINCIPAL COMPONENTS FROM THE DATA AND THE EIGENVECTORS.
!
!   IN ORDER TO COMPUTE NORMALIZED PCs, SET singval(:m) = sqrt(eigval(:m)),
!   OTHERWISE SET singval(:) = one .
!
    singval(:) = one
!
!   THE OPTIONAL PARAMETERS xmean AND xstd ARE REQUIRED IF THE EIGENVECTORS
!   HAVE BEEN COMPUTED FROM THE CORRRELATION MATRIX.
!
    call comp_pc_eof( x(:m,:p), eigvec(:m,:neig), singval(:neig), pc(:p,:neig),   &
                      xmean=mean(:m), xstd=std(:m)                                )
!
!   ON EXIT OF COMP_PC_EOF  :
!
!      pc(:p,:neig) CONTAINS THE FIRST neig PRINCIPAL COMPONENTS STORED COLUMNWISE.
!
!
!   CHECK FOR SMALL RESIDUALS OF THE EXPRESSION x_std**t*eigvec - pc
!   WHERE eigvec ARE THE EIGENVECTORS, x_std THE STANDARDIZED DATA
!   AND pc THE UNNORMALIZED PRINCIPAL COMPONENTS.
!
    x_std(:m,:p) = x(:m,:p) - spread( mean(:m) ,      dim=2, ncopies=p )
    x_std(:m,:p) = x_std(:m,:p)*spread( one/std(:m) , dim=2, ncopies=p ) 
!
    err_pc =  sum( abs(matmul(transpose(x_std),eigvec)-pc) )/sum( eigval(:neig) )
!
!   PRINT THE RESULT OF THE TEST.
!
    if ( err_pc<=sqrt( epsilon(err_pc) ) ) then
        write (prtunit,*) 'Example 1 of COMP_EOF2 is correct'
    else
        write (prtunit,*) 'Example 1 of COMP_EOF2 is incorrect'
    end if
!
!
! END OF PROGRAM ex1_comp_eof2
! ============================
!
end program ex1_comp_eof2
