program ex1_do_index
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutines DO_INDEX and REORDER
!   in module Sort_Procedures.
!                                                                              
! LATEST REVISION : 26/06/2006
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, false, arth, do_index, reorder
!   
!
! STRONG TYPING IMPOSED
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=100
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd), dimension(n) :: x
!
    integer(i4b)               :: i, j, k, i1, i2
    integer(i4b), dimension(n) :: y, indexx, indexy
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 1 of do_index'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   GENERATE RANDOM REAL DATA TO SORT.
!
    call random_number( x )
!
!   INITIALIZE PERMUTATION TO THE IDENTITY.
!
    y = arth( 1_i4b, 1_i4b, n ) 
!    
!   GENERATE A RANDOM ORDERING OF THE INTEGERS 1 TO n.
!   STARTING AT THE END, SWAP THE CURRENT LAST INDICATOR WITH ONE
!   RANDOMLY CHOSEN FROM THOSE PRECEEDING IT.

    do i = n, 2, -1
        j = 1 + i * x(i)
        if (j < i) then
            k = y(i)
            y(i) = y(j)
            y(j) = k
        end if
    end do
!
!   COMPUTE INDEX FOR EACH ARRAY.
!
    call do_index( x, indexx )
    call do_index( y, indexy )
!
!   EXAMPLE 1 : SORT THE REAL DATA IN ASCENDING ORDER 
!               BY MEANS OF THE INDEX .
!
    call reorder( indexx, x  )
!
!   CHECK THAT THE SORTED ARRAY IS NON-DECREASING.
!
    i1 = count( x(1:n-1) > x(2:n) )
!
!   EXAMPLE 2 : SORT THE INTEGER DATA IN DECREASING ORDER
!            BY MEANS OF THE INDEX .
!
    call reorder( indexy, y, ascending=false )
!
!   CHECK THAT THE SORTED ARRAY IS NON-INCREASING.
!
    i2 = count( y(1:n-1) < y(2:n) )
!
!   PRINT THE RESULT OF THE TESTS.
!
    if ( i1==0 .and. i2==0 ) then
        write (prtunit,*) 'Example 1 of DO_INDEX is correct'
    else
        write (prtunit,*) 'Example 1 of DO_INDEX is incorrect'
    end if
!
!
! END OF PROGRAM ex1_do_index
! ===========================
!
end program ex1_do_index
