program ex1_drawsample
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutines COMP_COR and DRAWSAMPLE
!   in modules Mul_Stat_Procedures and Random, respectively.
!                                                                              
! LATEST REVISION : 26/06/2006
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    USE Statpack, ONLY : i4b, stnd, lgl, true, comp_cor, drawsample, random_seed_, random_number_
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
! p       IS THE NUMBER OF OBSERVATIONS OF THE RANDOM VECTORS
! nrep    IS THE NUMBER OF SHUFFLES FOR THE PERMUTATION TEST
!
    INTEGER(i4b), PARAMETER :: prtunit=6, p=50, p1=26, p2=p, p3=p2-p1+1, nrep=999, nsample=2
!
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                        :: xyn
    REAL(stnd), DIMENSION(nsample)    :: xycor, prob, xycor2
    real(stnd), dimension(nsample,2)  :: xstat
    real(stnd), dimension(2)          :: ystat
    real(stnd), dimension(nsample,p)  :: x
    real(stnd), dimension(nsample,p3) :: x2
    real(stnd), dimension(p)          :: y
    real(stnd), dimension(p3)         :: y2
!
    integer(i4b)                     :: i
    integer(i4b), dimension(p)       :: pop
    integer(i4b), dimension(nsample) :: nge
!
    logical(lgl) :: first, last
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 1 of drawsample'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   INITIALIZE THE RANDOM GENERATOR.
!
    call random_seed_()
!
!   GENERATE A RANDOM UNIFORM OBSERVATION VECTOR y .
!
    call random_number_( y(:p) )
!
!   GENERATE A RANDOM UNIFORM OBSERVATION ARRAY x .
!
    call random_number_( x(:nsample,:p) )
!
!   COMPUTE THE CORRELATIONS BETWEEN x AND y
!   FOR THE p2-p1+1 LAST  OBSERVATIONS .
!
    first = true
    last  = true
!
    call comp_cor( x(:nsample,p1:p2), y(p1:p2), first, last, xstat(:nsample,:2), ystat(:2),    &
                   xycor(:nsample), xyn )
!
!   ON EXIT OF COMP_COR WHEN last=true :
!
!      xstat(:nsample,1)     CONTAINS THE MEAN VALUES OF THE OBSERVATION ARRAY x(:nsample,p1:p2).
!
!      xstat(:nsample,2)     CONTAINS THE VARIANCES OF THE OBSERVATION ARRAY x(:nsample,p1:p2).
!
!      ystat(1)              CONTAINS THE MEAN VALUE OF THE OBSERVATION VECTOR  y(p1:p2).
!
!      ystat(2)              CONTAINS THE VARIANCE OF THE OBSERVATION VECTOR  y(p1:p2).
!
!      xycor(:nsample)       CONTAINS THE CORRELATION COEFFICIENTS BETWEEN x(:nsample,p1:p2) AND y(p1:p2).
!
!      xyn                   CONTAINS THE NUMBER OF OBSERVATIONS IN THE DATA ARRAYS
!                            x(:nsample,p1:p2) AND  y(p1:p2) (xyn=real(p2-p1+1,stnd) ).
!
!
!   NOW COMPUTE A PERMUTATION TEST OF THE CORRELATION BETWEEN x AND y WITH
!   SUBROUTINES drawsample  AND comp_cor WITH nrep SHUFFLES .
!
    nge(:nsample) = 1
!
    do i=1, nrep
!
        call drawsample( p3, pop )
!
        x2(:nsample,:p3) = x(:nsample,pop(:p3))
        y2(:p3)          = y(pop(:p3))
!
        call comp_cor( x2(:nsample,:p3), y2(:p3), first, last, xstat(:nsample,:2), ystat(:2),    &
                       xycor2(:nsample), xyn )
!
        where( abs( xycor2(:nsample) )>= abs( xycor(:nsample) ) ) nge(:nsample) = nge(:nsample) + 1
!
    end do 
!
!   COMPUTE THE SIGNIFICANCE LEVELS.
!
    prob(:nsample) = real( nge(:nsample), stnd )/real( nrep+1, stnd )
!
    WRITE (prtunit,*) 'Correlations  = ', xycor(:nsample)
    WRITE (prtunit,*) 'Probabilities = ', prob(:nsample)
!
!
!
! END OF PROGRAM ex1_drawsample
! ==============================
!
end program ex1_drawsample
