program ex1_eigvalues
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of function EIGVALUES 
!   in module Eig_Procedures .
!                                                                              
! LATEST REVISION : 26/06/2006
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : lgl, i4b, stnd, svd_cmp, eigvalues, eigval_sort
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=1000
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)        :: a(n,n), a2(n,n), d(n), s(n)
!
    integer(i4b)      :: i
!
    logical(lgl)      :: failure
!   
    character         :: sort='d'
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter  :: name_proc='Example 1 of eigvalues'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   GENERATE A RANDOM DATA MATRIX AND FROM IT
!   A SELF-ADJOINT MATRIX.
!
    call random_number( a2 )
    a = a2 + transpose( a2 )
!
!   COMPUTE THE EIGENVALUES OF THE SELF-ADJOINT MATRIX.
!
    d = eigvalues( a )
!
!   SORT ABSOLUTE VALUES OF EIGENVALUES .
!
    d = abs( d )
    call eigval_sort( sort, d )
!
!   FOR COMPARISON, COMPUTE THE SINGULAR VALUES.
!
    call svd_cmp( a, s, failure, sort=sort )
!
!   CHECK THE RESULTS: MAGNITUDE OF EIGENVALUES SHOULD EQUAL THE SINGULAR VALUES.
!
    if ( .not. failure ) then
        if ( sum(abs(d-s))<=sqrt(epsilon(s))*maxval(abs(s)) ) then
            write (prtunit,*) 'Example 1 of EIGVALUES is correct'
        else
            write (prtunit,*) 'Example 1 of EIGVALUES is incorrect'
        end if
    else
        write (prtunit,*) 'Example 1 of EIGVALUES is not done'
    end if
!
!
! END OF PROGRAM ex1_eigvalues
! ============================
!
end program ex1_eigvalues
