program ex1_fastgivens_mat_left
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutine FASTGIVENS_MAT_LEFT 
!   in module Giv_Procedures . 
!                                                                              
! LATEST REVISION : 26/06/2006
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, one, fastgivens_mat_left
#ifdef _MATMUL
    use Statpack, only : matmul=>matmul2
#endif
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, m=1000, n=500, np1=n+1
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)               :: err
    real(stnd), dimension(m) :: a(m,n), syst(m,np1), x(n), b, res, d
!
    integer(i4b) :: i
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 1 of fastgivens_mat_left'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   GENERATE A RANDOM COEFFICIENT MATRIX a .
!
    call random_number( a )
!
!   GENERATE A RANDOM RIGHT HAND-SIDE VECTOR b .
!
    call random_number( b )
!
!   MAKE A COPY OF COEFFICIENT MATRIX a(:m,:n) .
!
    syst(:m,:n) = a(:m,:n)
!
!   MAKE A COPY OF RIGHT HAND-SIDE VECTOR b(:m) .
!
    syst(:m,np1) = b(:m)
!
!   EXAMPLE : LINEAR LEAST SQUARES SYSTEM.
!
!   COMPUTE SOLUTION VECTOR FOR LINEAR LEAST SQUARES SYSTEM
!
!             a(:m,:n)*x(:n)=b(:m) .
!
    d(:m) = one
!
!   TRANSFORM THE MATRIX a TO UPPER TRAPEZOIDAL FORM BY APPLYING 
!   A SERIE OF FAST GIVENS PLANE ROTATIONS ON THE ROWS OF a AND APPLY
!   THE ROTATIONS TO b .
!
    call fastgivens_mat_left( syst(:m,:np1), d(:m) )
!    
!   SOLVE THE n BY n UPPER TRIANGULAR SYSTEM.
!    
    do i = n, 1, -1
        x(i) = syst(i,np1)/syst(i,i)
        syst(1:i-1,np1) = syst(1:i-1,np1) - x(i)*syst(1:i-1,i)
    end do
!
!   CHECK THAT THE RESIDUALS ARE ORTHOGONAL TO THE COLUMN VECTORS OF a .
!
    res(:m) = b(:m) - matmul( a(:m,:n), x(:n) )
    err = sum(abs(matmul(res(:m)  ,a(:m,:n))) )/ sum( abs(a(:m,:n)) )
!
!   PRINT THE RESULT OF THE TEST.
!
    if ( err<=sqrt(epsilon(err)) ) then
        write (prtunit,*) 'Example 1 of FASTGIVENS_MAT_LEFT is correct'
    else
        write (prtunit,*) 'Example 1 of FASTGIVENS_MAT_LEFT is incorrect'
    end if
!
!
! END OF PROGRAM ex1_fastgivens_mat_left
! ======================================
!
end program ex1_fastgivens_mat_left
