program ex1_fastgivens_mat_right
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutine FASTGIVENS_MAT_RIGHT 
!   in module Giv_Procedures . 
!                                                                              
! LATEST REVISION : 26/06/2006
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, one, fastgivens_mat_right
#ifdef _MATMUL
    use Statpack, only : matmul=>matmul2
#endif
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, m=1000, n=500, np1=n+1
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)               :: err
    REAL(stnd), DIMENSION(m) :: a(n,m), syst(np1,m), x(n), b, res, d
!
    integer(i4b) :: i
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 1 of fastgivens_mat_right'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   GENERATE A RANDOM COEFFICIENT MATRIX a .
!
    call random_number( a )
!
!   GENERATE A RANDOM RIGHT HAND-SIDE VECTOR b .
!
    call random_number( b )
!
!   MAKE A COPY OF COEFFICIENT MATRIX a(:n,:m) .
!
    syst(:n,:m) = a(:n,:m)
!
!   MAKE A COPY OF RIGHT HAND-SIDE VECTOR b(:m) .
!
    syst(np1,:m) = b(:m)
!
!   EXAMPLE : LINEAR LEAST SQUARES SYSTEM.
!
!   COMPUTE SOLUTION VECTOR FOR LINEAR LEAST SQUARES SYSTEM
!
!             x(:n)*a(:n,:m)*=b(:m) .
!
    d(:m) = one
!
!   TRANSFORM THE MATRIX a TO UPPER TRAPEZOIDAL FORM BY APPLYING 
!   A SERIE OF FAST GIVENS PLANE ROTATIONS ON THE ROWS OF a AND APPLY
!   THE ROTATIONS TO b .
!
    call fastgivens_mat_right( syst(:np1,:m), d(:m) )
!    
!   SOLVE THE n BY n LOWER TRIANGULAR SYSTEM.
!    
    do i = n, 1, -1
        x(i) = ( syst(np1,i) - dot_product(syst(i+1:n,i),x(i+1:n)) )/syst(i,i)
    end do
!
!   CHECK THAT THE RESIDUALS ARE ORTHOGONAL TO THE COLUMN VECTORS OF a .
!
    res(:m) = b(:m) - matmul( x(:n), a(:n,:m) )
    err = sum(abs(matmul(a(:n,:m),res(:m))) )/ sum( abs(a(:n,:m)) )
!
!   PRINT THE RESULT OF THE TEST.
!
    if ( err<=sqrt(epsilon(err)) ) then
        write (prtunit,*) 'Example 1 of FASTGIVENS_MAT_RIGHT is correct'
    else
        write (prtunit,*) 'Example 1 of FASTGIVENS_MAT_RIGHT is incorrect'
    end if
!
!
! END OF PROGRAM ex1_fastgivens_mat_right
! =======================================
!
end program ex1_fastgivens_mat_right
