program ex1_fft
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutine FFT
!   in module FFT_Procedures .
!                                                                              
! LATEST REVISION : 15/06/2018
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, one, true, false, init_fft, fft, end_fft,  &
                              merror, allocate_error
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=1000024
!
    character(len=*), parameter :: name_proc='Example 1 of fft'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    integer      :: iok, istart, iend, irate, imax, itime
!
    real(stnd)                            :: err, eps, elapsed_time
    real(stnd), dimension(:), allocatable :: y
!
    complex(stnd), dimension(:), allocatable :: a, c
!   
!   
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   EXAMPLE 1 : FORWARD AND BACKWARD FFTS OF A COMPLEX SEQUENCE.
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = sqrt( epsilon( err ) )
!
!   ALLOCATE WORK ARRAYS.
!
    allocate( y(2_i4b*n), a(n), c(n), stat=iok )
!
    if ( iok/=0 ) then
        call merror( name_proc//allocate_error )
    end if
!
!   GENERATE A RANDOM NUMBER COMPLEX SEQUENCE.
!
    call random_number(y)
!
    a(:) = cmplx( y(1_i4b:n), y(n+1_i4b:2_i4b*n), kind=stnd)
!
!   SAVE THE COMPLEX SEQUENCE.
!
    c(:) = a(:)
!
!   INITIALIZE THE FFT SUBROUTINE.
!
    call init_fft( n )
!
!   START TIMING THE COMPUTATIONS.
!
    call system_clock( count_rate=irate, count_max=imax  )
    call system_clock( count=istart )
!
!   TRANSFORM AND THEN INVERT THE SEQUENCE BACK.
!
    call fft( a(:), forward=true  )
    call fft( a(:), forward=false )
!
!   STOP THE TIMER.
!
    call system_clock( count=iend )
!
    itime = iend - istart
    if ( iend<istart ) then
        itime = itime + imax   
    end if
!
    elapsed_time = real( itime, stnd )/real( irate, stnd )
!
    call end_fft()
!
!   CHECK THAT INVERSE(TRANSFORM(SEQUENCE)) = SEQUENCE.
!
    err = maxval(abs(c(:)-a(:)))/maxval(abs(c(:)))
!
!   DEALLOCATE WORK ARRAYS.
!
    deallocate( y, a, c )
!
    if ( err<=eps  ) then
        write (prtunit,*) name_proc//' is correct'
    else
        write (prtunit,*) name_proc//' is incorrect'
    end if
!
    write (prtunit,*)
    write (*,'(a,i8,a,0pd12.4,a)')    &
      'The elapsed time for computing the forward and backward FFTs of a complex sequence of size ', &
       n, ' is', elapsed_time, ' seconds'
!
!
! END OF PROGRAM ex1_fft
! ======================
!
end program ex1_fft
