program ex1_normal_random_number_
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutine NORMAL_RANDOM_NUMBER_ and
!   function NORMAL_RAND_NUMBER in module Random.
!                                                                            
! LATEST REVISION : 23/11/2016
!                                               
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd,  merror, allocate_error, random_seed_,  &
                         normal_rand_number, normal_random_number_
!
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n1=10**4, n2=10**4
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd) :: elapsed_time
    real(stnd), allocatable, dimension(:,:) :: real_mat
!
    integer(i4b) :: i, j
    integer      :: iok, istart, iend, irate, imax, itime
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 1 of normal_random_number_'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   ALLOCATE WORK ARRAY.
!
    allocate( real_mat(n1,n2), stat=iok )
!
    if ( iok/=0 ) then
        call merror( name_proc//allocate_error )
    end if
!
!   SELECT THE UNIFORM RANDOM GENERATOR TO BE USED.
!
    call random_seed_( alg=2 )
!
!   RESET THE SEEDS USED BY THE UNIFORM RANDOM GENERATOR.
!
    call random_seed_()
!
    call system_clock( count_rate=irate, count_max=imax  )
!
!   START TIMING THE COMPUTATIONS.
!
    call system_clock( count=istart )
!
!   GENERATE A NORMAL RANDOM REAL MATRIX USING FUNCTION normal_rand_number().
!
    do i = 1_i4b, n2
!
        do j = 1_i4b, n1
!
             real_mat(j,i) = normal_rand_number( )
!
        end do
!
    end do
!
!   STOP THE TIMER.
!
    call system_clock( count=iend )
!
    itime = iend - istart
    if ( iend<istart ) then
        itime = itime + imax   
    end if
!
    elapsed_time = real( itime, stnd )/real( irate, stnd )
!
    write (*,'(a,i10,a,0pd12.4,a)')              &
      'The elapsed time for generating ', n1*n2, &
      ' random normal real numbers with function normal_rand_number() is',  &
      elapsed_time, ' seconds'
!
!   START TIMING THE COMPUTATIONS.
!
    call system_clock( count=istart )
!
!   GENERATE A NORMAL RANDOM REAL MATRIX USING VECTOR
!   FORM OF SUBROUTINE normal_random_number_.
!
    do i = 1_i4b, n2
!
        call normal_random_number_( real_mat(:n1,i) )
!
    end do
!
!   STOP THE TIMER.
!
    call system_clock( count=iend )
!
    itime = iend - istart
    if ( iend<istart ) then
        itime = itime + imax   
    end if
!
    elapsed_time = real( itime, stnd )/real( irate, stnd )
!
    write (prtunit,*) 
    write (*,'(a,i10,a,0pd12.4,a)')              &
      'The elapsed time for generating ', n1*n2, &
      ' random normal real numbers with vector form of subroutine normal_random_number_ is',  &
      elapsed_time, ' seconds'
!
!   START TIMING THE COMPUTATIONS.
!
    call system_clock( count=istart )
!
!   GENERATE A NORMAL RANDOM REAL MATRIX USING MATRIX
!   FORM OF SUBROUTINE normal_random_number_.
!
    call normal_random_number_( real_mat(:n1,:n2) )
!
!   STOP THE TIMER.
!
    call system_clock( count=iend )
!
    itime = iend - istart
    if ( iend<istart ) then
        itime = itime + imax   
    end if
!
    elapsed_time = real( itime, stnd )/real( irate, stnd )
!
    write (prtunit,*) 
    write (*,'(a,i10,a,0pd12.4,a)')              &
      'The elapsed time for generating ', n1*n2, &
      ' random normal real numbers with matrix form of subroutine normal_random_number_ is',  &
      elapsed_time, ' seconds'
!
!   DEALLOCATE WORK ARRAY.
!
    deallocate( real_mat )
!
!
! END OF PROGRAM ex1_normal_random_number_
! ========================================
!
end program ex1_normal_random_number_
