program ex1_probbeta
!
!
! Purpose
! =======
!
!   This program is intended to illustrate the use of functions PROBBETA, PINVSTUDENT
!   in module Prob_Procedures .
!                                                                              
!                                                                              
! LATEST REVISION : 22/02/2013
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : lgl, i4b, stnd, half, probbeta, pinvstudent
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=5000, m=10000
!
    real(stnd), parameter :: eps = 1.0e-4_stnd 
!
    character(len=*), parameter :: name_proc='Example 1 of probbeta'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd), dimension(n,m) :: p, p2, t, x
    real(stnd)                 :: err, df, a
!   
!   
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   GENERATE THE DEGREES OF FREEDOM OF STUDENT T-DISTRIBUTION .
!
    df = 50_stnd
!
!   GENERATE A RANDOM PROBABILITY MATRIX p .
!
    call random_number( p(:n,:m) )
!
!   COMPUTE THE TWO-TAIL QUANTILES T OF STUDENT T-DISTRIBUTION
!   WITH df DEGREES OF FREEDOM.
!
    t(:n,:m) = pinvstudent( p(:n,:m), df )
!
!   RECOMPUTE THE PROBABILITIES FROM THE QUANTILES
!   WITH probbeta FUNCTION.
!
    x(:n,:m) =  df/(df+ t(:n,:m)*t(:n,:m) )
    a = half*df
!
    p2(:n,:m) = probbeta( x(:n,:m), a, half )
!
!   CHECK THAT p AND p2 AGREE.
!
    err = maxval( abs( p(:n,:m) - p2(:n,:m) ) )
!
!   PRINT THE RESULT OF THE TEST.
!
    if ( err<=eps ) then
        write (prtunit,*) name_proc//' is correct'
    else
        write (prtunit,*) name_proc//' is incorrect'
    end if
!
!
! END OF PROGRAM ex1_probbeta
! ===========================
!
end program ex1_probbeta
