program ex1_probn
!
!
! Purpose
! =======
!
!   This program is intended to illustrate the use of functions PROBN and PINVN
!   in module Prob_Procedures .
!                                                                              
!                                                                              
! LATEST REVISION : 15/02/2013
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, false, probn, pinvn
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=5000, m=10000
!
    real(stnd), parameter :: eps = 1.0e-4_stnd 
!
    character(len=*), parameter :: name_proc='Example 1 of probn'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd), dimension(n,m) :: p, p2, x
    real(stnd)                 :: err
!   
!   
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   GENERATE A RANDOM PROBABILITY MATRIX p .
!
    call random_number( p(:n,:m) )
!
!   COMPUTE THE NORMAL DEVIATES CORRESPONDING TO LOWER TAIL AREAS OF P .
!
    x(:n,:m) = pinvn( p(:n,:m) ) 
!
!   RECOMPUTE THE PROBABILITIES FROM THE NORMAL DEVIATES .
!
    p2(:n,:m) = probn( x(:n,:m), upper=false )
!
!   CHECK THAT p AND p2 AGREE.
!
    err = maxval( abs( p(:n,:m) - p2(:n,:m) ) )
!
!   PRINT THE RESULT OF THE TEST.
!
    if ( err<=eps ) then
        write (prtunit,*) name_proc//' is correct'
    else
        write (prtunit,*) name_proc//' is incorrect'
    end if
!
!
! END OF PROGRAM ex1_probn
! ========================
!
end program ex1_probn
