program ex1_real_fft_forward
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutine REAL_FFT_FORWARD 
!   and REAL_FFT_BACKWARD in module FFT_Procedures .
!                                                                              
! LATEST REVISION : 15/06/2018
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, real_fft_forward, real_fft_backward, merror, allocate_error
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT, n IS THE LENGTH OF THE TIME SERIES.
!
    integer(i4b), parameter :: prtunit=6, n=4000
!
    character(len=*), parameter :: name_proc='Example 1 of real_fft_forward'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                            :: err, eps, elapsed_time
    real(stnd), dimension(:), allocatable :: y, y2, yi, yr
!
    integer(i4b) :: nd2p1
    integer      :: iok, istart, iend, irate, imax, itime
!   
!   
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   EXAMPLE 1 : FORWARD AND BACWARD FFTS OF A REAL SEQUENCE BY THE GOERTZEL METHOD.
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = sqrt( epsilon( err ) )
!
    nd2p1 = (n/2) + 1
!
!   ALLOCATE WORK ARRAYS.
!
    allocate( y(n), y2(n), yi(nd2p1), yr(nd2p1), stat=iok )
!
    if ( iok/=0 ) then
        call merror( name_proc//allocate_error )
    end if
!
!   GENERATE A RANDOM REAL NUMBER SEQUENCE.
!
    call random_number( y(:n) )
!
!   START TIMING THE COMPUTATIONS.
!
    call system_clock( count_rate=irate, count_max=imax  )
    call system_clock( count=istart )
!
!   TRANSFORM THE REAL SEQUENCE.
!
    call real_fft_forward( y(:n), yr(:nd2p1), yi(:nd2p1)  )
!
!   INVERT THE SEQUENCE BACK.
!
    call real_fft_backward( yr(:nd2p1), yi(:nd2p1), y2(:n) )
!
!   STOP THE TIMER.
!
    call system_clock( count=iend )
!
    itime = iend - istart
    if ( iend<istart ) then
        itime = itime + imax   
    end if
!
    elapsed_time = real( itime, stnd )/real( irate, stnd )
!
!   CHECK THAT INVERSE(TRANSFORM(SEQUENCE)) = SEQUENCE.
!
    err = maxval(abs(y(:n)-y2(:n)))/maxval(abs(y(:n)))
!
!   DEALLOCATE ARRAYS.
!
    deallocate( y, y2, yi, yr )
!
    if ( err<=eps  ) then
        write (prtunit,*) name_proc//' is correct'
    else
        write (prtunit,*) name_proc//' is incorrect'
    end if
!
    write (prtunit,*)
    write (*,'(a,i8,a,0pd12.4,a)')    &
      'The elapsed time for computing the forward and backward FFTs of a real sequence of size ', &
       n, ' is', elapsed_time, ' seconds'
!
!
! END OF PROGRAM ex1_real_fft_forward
! ===================================
!
end program ex1_real_fft_forward
