program ex1_rtsw
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of functions RTSW and TIME_TO_STRING
!   in module Time_Procedures .
!                                                                              
! LATEST REVISION : 06/07/2006
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, extd, rtsw, time_to_string
!   
!
! STRONG TYPING IMPOSED
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT.
!
    integer(i4b), parameter :: prtunit=6
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(extd)   :: tim1, tim2
!
    integer(i4b)  :: i, j
!
    character(len=13) :: string
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 1 of rtsw'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   FUNCTION rtsw CAN BE USED TO COMPUTE THE TIME LAPSE BETWEEN
!   FUNCTIONS CALLS ACCORDING TO THE SYSTEM (WALL) CLOCK.
!
!   FUNCTION rtsw OBTAINS, FROM THE INTRINSIC ROUTINE DATE_AND_TIME,
!   THE CURRENT DATE AND TIME. THESE VALUES ARE THEN CONVERTED TO SECONDS 
!   AND RETURNED AS AN EXTENDED PRECISION REAL VALUE.
!
!   THIS FUNCTION WORKS ACROSS MONTH AND YEAR BOUNDARIES, BUT WILL NOT WORK
!   PROPERLY WITH OPENMP (USE FUNCTION elapsed_time IN THIS CASE).
!
!   SINCE THIS ROUTINE USES THE SYSTEM CLOCK, THE ELAPSED TIME COMPUTED
!   WITH THIS ROUTINE MAY NOT (PROBABLY WON'T BE IN A MULTI-TASKING OS)
!   AN ACCURATE REFLECTION OF THE NUMBER OF CPU CYCLES REQUIRED TO
!   PERFORM A CALCULATION. THEREFORE CARE SHOULD BE EXERCISED WHEN
!   USING THIS TO PROFILE A CODE.
!
!   A TYPICAL USE OF THIS FUNCTION IS AS FOLLOWS :
!
    tim1 = rtsw()
    j    = 0
    do i=1, 1000000000
        j = j + 1
    end do
    tim2 = rtsw()
!
!   CONVERT THE ELAPSED TIME tim2-tim1 TO A STRING FORMAT FOR PRINTING AS
!
!           'milliseconds.seconds.minutes.hours'
!
!   WITH SUBROUTINE time_to_string .
!
    string = time_to_string( tim2-tim1 )
!
!   PRINT THE RESULT.
!
    write (prtunit, *)  " Elapsed Time (s): " // string //  " => milliseconds.seconds.minutes.hours "
!
!
! END OF PROGRAM ex1_rtsw
! =======================
!
end program ex1_rtsw
