program ex1_singvalues
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of function SINGVALUES 
!   in module SVD_Procedures . 
!                                                                              
! LATEST REVISION : 06/07/2006
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : lgl, i4b, stnd, svd_cmp, singvalues
#ifdef _MATMUL
    use Statpack, only : matmul=>matmul2
#endif
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit = 6, m=1000, n=100
!   
!   
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                 :: err
    real(stnd), dimension(n)   :: s, s2
    real(stnd), dimension(m,n) :: a, u
    real(stnd), dimension(n,n) :: v
!
    logical(lgl)   :: failure
!   
    character      :: sort='a'
!   
!
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 1 of singvalues'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   GENERATE A RANDOM DATA MATRIX.
!
    call random_number( a )
!
!   SAVE RANDOM DATA MATRIX .
!
    u(:m,:n) = a(:m,:n)
!
!   COMPUTE FULL SVD OF RANDOM DATA MATRIX.
!
    call svd_cmp( u, s, failure, v, sort=sort )
!
!   CHECK FOR SMALL RESIDUALS OF THE EXPRESSION a*v(:,:k) - u(:,:k)*s.
!
    err =  sum( abs(matmul(a,v) - u*spread(s,dim=1,ncopies=m)) )/sum( abs(s) )
!
    if ( err<=sqrt(epsilon(err)) .and. .not.failure ) then
!
!       COMPUTE ONLY SINGULAR VALUES OF RANDOM DATA MATRIX.
!
        s2 = singvalues( a, sort=sort )
!
!       CHECK THE RESULTS WITH THE PREVIOUS COMPUTATIONS.
!
        if ( sum(abs(s2-s))<=sqrt(epsilon(err))*maxval(abs(s)) ) then
            write (prtunit,*) 'Example 1 of SINGVALUES is correct'
        else
            write (prtunit,*) 'Example 1 of SINGVALUES is incorrect'
        end if
    else
        write (prtunit,*) 'Example 1 of SINGVALUES is not done'
    end if
!
!
! END OF PROGRAM ex1_singvalues
! =============================
!
end program ex1_singvalues

