program ex1_sym_inv
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of function SYM_INV
!   in module Lin_Procedures . 
!                                                                              
! LATEST REVISION : 23/09/2014
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    USE Statpack, ONLY : lgl, i4b, stnd, true, false, zero, one, sym_inv, merror, allocate_error
#ifdef _MATMUL
    use Statpack, only : matmul=>matmul2
#endif
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=4000, m=4000
!
    character(len=*), parameter :: name_proc='Example 1 of sym_inv'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                              :: err, eps, elapsed_time
    real(stnd), dimension(:,:), allocatable :: a, ata, atainv, res
!
    integer(i4b) :: j
    integer      :: iok, istart, iend, irate
!
    logical(lgl) :: do_test, upper, failure
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   EXAMPLE 1 : INVERSE OF A REAL SYMMETRIC DEFINITE POSITIVE MATRIX .
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = sqrt( epsilon( err ) )
    err = zero
!
    do_test = false
    upper   = true
!
!   ALLOCATE WORK ARRAYS.
!
    allocate( a(m,n), ata(n,n), atainv(n,n), stat=iok )
!
    if ( iok/=0 ) then
        call merror( name_proc//allocate_error )
    end if
! 
!   GENERATE A RANDOM DATA MATRIX a .
!
    call random_number( a )
!
!   GENERATE A n-BY-n SYMMETRIC POSITIVE DEFINITE MATRIX From a .
!
    ata = matmul( transpose(a), a )
!
!   START TIMING THE COMPUTATIONS.
!
    call system_clock( count=istart, count_rate=irate )
!
!   COMPUTE THE MATRIX INVERSE OF ata WITH FUNCTION sym_inv.
!
    atainv = sym_inv( ata, upper=upper )
!
!   STOP THE TIMER.
!
    call system_clock( count=iend )
!
    elapsed_time = real( iend - istart, stnd )/real( irate, stnd )
!
    if ( do_test ) then
!
!       ALLOCATE WORK ARRAY .
!
        allocate( res(n,n), stat=iok )
!
        if ( iok/=0 ) then
            call merror( name_proc//allocate_error )
        end if
!
!       COMPUTE ata TIMES ITS INVERSE - IDENTITY.
!
        res = matmul( ata, atainv )
!
        do j = 1_i4b, n
            res(j,j) = res(j,j) - one
        end do
!
!       CHECK THE RESULTS FOR SMALL RESIDUALS.
!
        err = sum( abs(res) ) / sum( abs(ata) )
!
!       DEALLOCATE WORK ARRAY.
!
        deallocate( res )
!
    end if
!
!   DEALLOCATE WORK ARRAYS.
!
    deallocate( a, ata, atainv )
!
    if ( err<=eps .and. .not.failure ) then
        write (prtunit,*) name_proc//' is correct'
    else
        write (prtunit,*) name_proc//' is incorrect'
    end if
!
    write (prtunit,*) 
    write (*,'(a,i5,a,0pd12.4,a)')    &
      'The elapsed time for computing the inverse of a positive definite symmetric matrix of size ', &
       n, ' is', elapsed_time, ' seconds'
!
!
! END OF PROGRAM ex1_sym_inv
! ==========================
!
end program ex1_sym_inv
