program ex1_symlin_filter2
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutines/functions LP_COEF, HP_COEF,
!   SYMLIN_FILTER2 in module Time_Series_Procedures.
!                                                                              
! LATEST REVISION : 12/11/2007
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lp_coef, hp_coef, symlin_filter2
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT, n IS THE LENGTH OF THE TIME SERIES, k IS THE NUMBER OF TERMS OF THE LANCZOS FILTER.
!
    integer(i4b), parameter :: prtunit=6, n=2000, k=21
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                     :: err
    real(stnd), dimension(n)       :: y, y2, y3
    real(stnd), dimension(k)       :: coefl, coefh
!
    integer(i4b) :: pc
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 1 of symlin_filter2'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   GENERATE A RANDOM REAL NUMBER SEQUENCE OF LENGTH n .
!
    call random_number( y(:n) )
!
!   SAVE THE REAL RANDOM NUMBER ARRAY.
!
    y2(:n) = y(:n)
    y3(:n) = y(:n)
!
!   DETERMINE THE CUTOFF PERIOD.
!
    pc  = 18
!
!   FUNCTION lp_coef COMPUTES THE K-TERM LEAST SQUARES APPROXIMATION TO AN
!   -IDEAL- LOW PASS FILTER WITH CUTOFF PERIOD PL (EG CUTOFF FREQUENCY FC =  1/PL).
!
    coefl(:k) = lp_coef( PL=pc, K=k )
!
!   FUNCTION hp_coef COMPUTES THE K-TERM LEAST SQUARES APPROXIMATION TO AN
!   -IDEAL- HIGH PASS FILTER WITH CUTOFF PERIOD PH (EG CUTOFF FREQUENCY FC =  1/PH).
!
    coefh(:k) = hp_coef( PH=pc, K=k )
!
!   PL AND  PH ARE EXPRESSED IN NUMBER OF POINTS, i.e. PL OR PH =6(18) CORRESPONDS TO PERIODS
!   OF 1.5 YRS FOR QUATERLY(MONTHLY) DATA.
!
!   SUBROUTINE symlin_filter2 PERFORMS A SYMMETRIC FILTERING OPERATION ON AN INPUT TIME
!   SERIES (EG THE ARGUMENT VEC) WITH THE ARRAY OF SYMMETRIC LINEAR FILTER COEFFICIENTS COEF(:).
!
!   NO DATA POINTS WILL BE LOST, HOWEVER (size(COEF)-1)/2 OBSERVATIONS
!   AT EACH END OF THE TIME SERIES ARE AFFECTED BY END EFFECTS.
!
    call symlin_filter2( VEC=y2(:n), COEF=coefl(:k) )
!
    call symlin_filter2( VEC=y3(:n), COEF=coefh(:k)  )
!
!   NOW RECONSTRUCT THE ORIGINAL TIME SERIES FROM THE TWO FILTERED TIME SERIES.
!
    y2(:n) = y2(:n) + y3(:n)
!
!   TEST THE ACCURACY OF THE RECONSTRUCTION.
!
    err = maxval(abs(y(:n)-y2(:n)))/maxval(abs(y(:n)))
!
    if ( err<=sqrt(epsilon(err))  ) then
        write (prtunit,*) name_proc//' is correct'
    else
        write (prtunit,*) name_proc//' is incorrect'
    end if
!
!
! END OF PROGRAM ex1_symlin_filter2
! =================================
!
end program ex1_symlin_filter2
