program ex1_transpose2
!
!
! Purpose
! =======
!
!   This program illustrates the use of function TRANSPOSE2
!   in module Module_Utilities and compares its efficiency with the intrinsic TRANSPOSE function. 
!                                                                              
! LATEST REVISION : 21/12/2015
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : lgl, i4b, stnd, true, false, transpose2, merror, allocate_error
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=10000, m=10000
!
    character(len=*), parameter :: name_proc='Example 1 of transpose2'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                              :: elapsed_time1, elapsed_time2
    real(stnd), dimension(:,:), allocatable :: a, a2, at
!
    integer :: iok, istart, iend, irate
!
    logical(lgl)                              :: do_test, failure
    logical(lgl), dimension(:,:), allocatable :: test
!   
!   
! EXECUTABLE STATEMENTS
! =====================
!
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   EXAMPLE 1 : TRANSPOSITION OF A REAL MATRIX.
!
    do_test = true
!
!   ALLOCATE WORK ARRAYS.
!
    allocate( a(n,m), at(m,n), stat=iok )
!
    if ( iok/=0 ) then
        call merror( name_proc//allocate_error )
    end if
!
!   GENERATE A n-by-m RANDOM REAL MATRIX a .
!
    call random_number( a(:n,:m) )
!
!   TRANSPOSE THE MATRIX WITH transpose2 FUNCTION.
!
!   START TIMING THE COMPUTATIONS.
!
    call system_clock( count=istart, count_rate=irate )
!
    at(:m,:n) = transpose2( a(:n,:m) )
!
!   STOP THE TIMER.
!
    call system_clock( count=iend )
!
    elapsed_time1 = real( iend - istart, stnd )/real( irate, stnd )
!
    if ( do_test ) then
!
!       ALLOCATE WORK ARRAYS .
!
        allocate( a2(n,m), test(n,m), stat=iok )
!
        if ( iok/=0 ) then
           call merror( name_proc//allocate_error )
        end if
!
        a2(:n,:m) = transpose2( at(:m,:n) )
!
!       CHECK THE RESULTS.
!
        test(:n,:m) = a(:n,:m) /= a2(:n,:m)
!
        failure = any( test(:n,:m) )
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( a2, test )
!
    end if
!
!   NOW TRANSPOSE THE MATRIX WITH INTRINSIC transpose FUNCTION.
!
!   START TIMING THE COMPUTATIONS.
!
    call system_clock( count=istart, count_rate=irate )
!
    at(:m,:n) = transpose( a(:n,:m) )
!
!   STOP THE TIMER.
!
    call system_clock( count=iend )
!
    elapsed_time2 = real( iend - istart, stnd )/real( irate, stnd )
!
!   DEALLOCATE WORK ARRAYS.
!
    deallocate( a, at )
!
!   CHECK AND PRINT THE RESULTS.
!
    if ( .not. failure ) then
        write (prtunit,*) name_proc//' is correct'
    else
        write (prtunit,*) name_proc//' is incorrect'
    end if
!
    write (prtunit,*) 
    write (*,'(a,i5,a,i5,a,0pd12.4,a)')    &
      'The elapsed time for transposing a real matrix of size ', n, ' by ', m,  &
       ' with transpose2() function is ', elapsed_time1, ' seconds'
!
    write (prtunit,*) 
    write (*,'(a,i5,a,i5,a,0pd12.4,a)')    &
      'The elapsed time for transposing a real matrix of size ', n, ' by ', m,  &
       ' with the intrinsic transpose() function is ', elapsed_time2, ' seconds'
!
!
! END OF PROGRAM ex1_transpose2
! =============================
!
end program ex1_transpose2
