program ex1_ymd_to_daynum
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of function YMD_TO_DAYNUM
!   in module Time_Procedures .
!                                                                              
!   See also program ex1_daynum_to_ymd.f90 .                                                      
!                                                                              
! LATEST REVISION : 06/07/2006
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, ymd_to_daynum, get_date
!   
!
! STRONG TYPING IMPOSED
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT.
!
    integer(i4b), parameter :: prtunit=6
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    integer(i4b) :: iyr, imon, iday, julday, iyr2, imon2, iday2, julday2
!
    character(len=11) :: date, date2
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 1 of ymd_to_daynum'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   GENERATE A DATE.
!
    iyr  = 1902
    imon = 11
    iday = 15
!
!   GENERATE ANOTHER DATE.
!
    iyr2  = 1982
    imon2 = 10
    iday2 = 22
!
!   CONVERTS GREGORIAN YEAR (iyr), MONTH (imon) AND DAY (iday) TO JULIAN DAY
!   NUMBER.
!
    julday = ymd_to_daynum( iyr, imon, iday )
!
!   FUNCTION ymd_to_daynum CONVERTS THE THREE INTEGERS iyr, imon AND iday STANDING FOR
!   YEAR, MONTH, DAY IN THE GREGORIAN CALENDAR PROMULGATED BY GREGORY XIII ON
!   FRIDAY, 15 OCTOBER 1582, IN THE CORRESPONDING JULIAN DAY NUMBER STARTING
!   WITH ymd_to_daynum=1 ON FRIDAY, 15 OCTOBER 1582.
!
!   NOTE THE GREGORIAN CALENDAR WAS ADOPTED IN OCT. 15, 1582, AND HENCE
!   THIS FUNCTION WILL NOT WORK PROPERLY FOR DATES EARLY THAN 10-15-1582.
!   
!   CONVERTS GREGORIAN YEAR (iyr2), MONTH (imon2) AND DAY (iday2) TO JULIAN DAY
!   NUMBER.
!
    julday2 = ymd_to_daynum( iyr2, imon2, iday2 )
!
!   THE NUMBER OF DAYS BETWEEN TWO DATES IS THE DIFFERENCE BETWEEN THEIR
!   JULIAN DAY. SO, ONE OF THE MOST USEFUL APPLICATIONS FOR THIS ROUTINE
!   IS TO COMPUTE THE NUMBER OF DAYS BETWEEN TWO DATES.
!
    call get_date( iyr, imon, iday, date )
    call get_date( iyr2, imon2, iday2, date2 )
!
    write (prtunit,*)      &
      'The number of days between ' // date2 // ' and ' // date // ' is ', julday2-julday
!
!
! END OF PROGRAM ex1_ymd_to_daynum
! ================================
!
end program ex1_ymd_to_daynum
