program ex2_comp_cor
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutine COMP_COR
!   in module Mul_Stat_Procedures .
!                                                                              
! LATEST REVISION : 06/07/2006
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, true, comp_cor
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=26, m=20, p=50
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                    :: err_xstat, err_ystat, err_cor, xyn, eps
    real(stnd), dimension(n,m)    :: xycor1, xycor2
    real(stnd), dimension(m,2)    :: ystat1, ystat2
    real(stnd), dimension(n,2)    :: xstat1, xstat2
    real(stnd), dimension(n,p)    :: x
    real(stnd), dimension(m,p)    :: y
!
    integer(i4b) :: i
!
    logical(lgl) :: first, last
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 2 of comp_cor'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
    eps = sqrt( epsilon(eps) )
!
!   GENERATE A RANDOM OBSERVATION ARRAY x .
!
    call random_number( x(:n,:p) )
!
!   GENERATE A RANDOM OBSERVATION ARRAY y .
!
    call random_number( y(:m,:p) )
!
!   COMPUTE THE CORRELATIONS BETWEEN x AND y
!   FOR THE p OBSERVATIONS .
!
    first = true
    last  = true
    call comp_cor( x(:n,:p), y(:m,:p), first, last,                    &
                   xstat1(:n,:2), ystat1(:m,:2), xycor1(:n,:m), xyn )
!
!   ON EXIT OF COMP_COR WHEN last=true :
!
!      xstat1(i,1) CONTAINS THE MEAN VALUES OF THE ARRAY SECTION x(i,:p).
!
!      xstat1(i,2) CONTAINS THE VARIANCES OF THE ARRAY SECTION x(i,:p).
!
!      ystat1(j,1) CONTAINS THE MEAN VALUE OF THE ARRAY SECTION y(j,:p).
!
!      ystat1(j,2) CONTAINS THE VARIANCE OF THE ARRAY SECTION y(j,:p).
!
!      xycor1(i,j) CONTAINS THE CORRELATION COEFFICIENT
!                  BETWEEN x(i,:p) AND y(j,:p).
!
!      xyn         CONTAINS THE NUMBER OF OBSERVATIONS IN THE DATA ARRAYS
!                  x(:n,:p) AND y(:m,:p) (xyn=real(p,stnd) ).
!
!   COMPUTE CORRELATIONS BETWEEN x AND y,
!   ITERATIVELY  FOR THE p OBSERVATIONS .
!
    do i = 1, p
!
        first = i==1
        last  = i==p
!
        call comp_cor( x(:n,i:i), y(:m,i:i), first, last,                      &
                       xstat2(:n,:2), ystat2(:m,:2), xycor2(:n,:m), xyn )
    end do
!
!   CHECK THAT THE TWO SETS OF STATISTICS AGREE.
!
    err_xstat = maxval( abs( ( xstat1-xstat2)/xstat1 ) )
    err_ystat = maxval( abs( ( ystat1-ystat2)/ystat1 ) )
    err_cor   = maxval( abs( xycor1-xycor2 ) )
!
    if ( max(err_xstat, err_ystat, err_cor )<=eps ) then
        write (prtunit,*) 'Example 2 of COMP_COR is correct'
    else
        write (prtunit,*) 'Example 2 of COMP_COR is incorrect'
    end if
!
!
! END OF PROGRAM ex2_comp_cor
! ===========================
!
end program ex2_comp_cor
