program ex2_comp_unistat
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutine COMP_UNISTAT
!   in module Stat_Procedures .
!                                                                              
! LATEST REVISION : 06/07/2006
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, true, comp_unistat
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! xmiss IS THE MISSING INDICATOR
!
    real(stnd), parameter  :: xmiss=-999.99_stnd
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=26, m=20, p=250
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                      :: err1, err2, eps
    real(stnd), dimension(n,m,7)    :: xstat1, xstat2
    real(stnd), dimension(n,m,p)    :: x
!
    integer(i4b)                  :: i
    integer(i4b), dimension(n,m)  :: xnobs1, xnobs2
!
    logical(lgl) :: first, last
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 2 of comp_unistat'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
    eps = sqrt( epsilon(eps) )
!
!   GENERATE A RANDOM OBSERVATION ARRAY x WITH MISSING VALUES, xmiss IS THE MISSING INDICATOR .
!
    call random_number( x )
!
    where( x(:,:,:)<=0.25_stnd )  x(:,:,:) = xmiss
!
!   COMPUTE THE STATISTICS OF x FOR THE p OBSERVATIONS .
!
    first = true
    last  = true
!
    call comp_unistat( x(:n,:m,:p), first, last, xstat1(:n,:m,:7), xmiss=xmiss, xnobs=xnobs1(:n,:m) )
!
!   ON EXIT, WHEN last=true, xstat1 CONTAINS THE FOLLOWING
!   STATISTICS ON ALL VARIABLES :
!
!        xstat1(:,:,1) CONTAINS THE MEAN VALUES.
!        xstat1(:,:,2) CONTAINS THE VARIANCES.
!        xstat1(:,:,3) CONTAINS THE STANDARD DEVIATIONS.
!        xstat1(:,:,4) CONTAINS THE COEFFICIENTS OF SKEWNESS.
!        xstat1(:,:,5) CONTAINS THE COEFFICIENTS OF KURTOSIS.
!        xstat1(:,:,6) CONTAINS THE MINIMA.
!        xstat1(:,:,7) CONTAINS THE MAXIMA.
!
!   ON EXIT,  xnobs(:,:) CONTAINS THE NUMBERS OF NON-MISSING OBSERVATIONS
!   ON ALL VARIABLES. xnobs NEEDS TO BE SPECIFIED ONLY ON THE LAST 
!   CALL TO comp_unistat (LAST=true).
!
!   COMPUTE THE STATISTICS OF x, ITERATIVELY FOR THE p OBSERVATIONS  .
!
    do i = 1, p
!
        first = i==1
        last  = i==p
!
        call comp_unistat( x(:n,:m,i:i), first, last, xstat2(:n,:m,:7), xmiss=xmiss, xnobs=xnobs2(:n,:m) )
    end do
!
!   CHECK THAT THE TWO SETS OF STATISTICS AGREE.
!
    err1 = maxval( abs( (xstat2(:,:,1:3)-xstat1(:,:,1:3))/xstat1(:,:,1:3) ) )
    err2 = maxval( abs( xstat2(:,:,4:7)-xstat1(:,:,4:7) ) )
!
    if ( max(err1, err2)<=eps  .and. all( xnobs2(:,:)==xnobs1(:,:) ) ) then
        write (prtunit,*) 'Example 2 of COMP_UNISTAT is correct'
    else
        write (prtunit,*) 'Example 2 of COMP_UNISTAT is incorrect'
    end if
!
!
! END OF PROGRAM ex2_comp_unistat
! ===============================
!
end program ex2_comp_unistat
