program ex2_probstudent
!
!
! Purpose
! =======
!
!   This program is intended to illustrate the use of functions PROBSTUDENT, PINVSTUDENT
!   in module Prob_Procedures .
!                                                                              
!                                                                              
! LATEST REVISION : 26/02/2013
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, false, two, c99, probstudent, pinvstudent
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=5000, m=10000
!
    real(stnd), parameter :: eps = 1.0e-3_stnd 
!
    character(len=*), parameter :: name_proc='Example 2 of probstudent'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd), dimension(n,m) :: p, p2, t, df
    real(stnd)                 :: err
!   
!   
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   GENERATE THE DEGREES OF FREEDOM OF STUDENT T-DISTRIBUTION .
!
    call random_number( df(:n,:m) )
!
    df(:n,:m) = c99*df(:n,:m)
    where( df(:n,:m)<two )  df(:n,:m) = df(:n,:m) + two
!
!   GENERATE A RANDOM PROBABILITY MATRIX p(:n,:m) .
!
    call random_number( p(:n,:m) )
!
!   COMPUTE THE TWO-TAIL QUANTILES t OF STUDENT T-DISTRIBUTION 
!   WITH df DEGREES OF FREEDOM CORRESPONDING TO AREAS OF p(:n,:m) .
!
    t(:n,:m) = pinvstudent( p(:n,:m), df(:n,:m) )
!
!   RECOMPUTE THE PROBABILITIES FROM THE QUANTILES
!   WITH probstudent FUNCTION.
!
    p2(:n,:m) = probstudent( t(:n,:m), df(:n,:m) )
!
!   CHECK THAT p AND p2 AGREE.
!
    err = maxval( abs( p(:n,:m) - p2(:n,:m) ) )
!
!   PRINT THE RESULT OF THE TEST.
!
    if ( err<=eps ) then
        write (prtunit,*) name_proc//' is correct'
    else
        write (prtunit,*) name_proc//' is incorrect'
    end if
!
!
! END OF PROGRAM ex2_probstudent
! ==============================
!
end program ex2_probstudent
