program ex2_symtrid_qri2
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutine SYMTRID_QRI2
!   in module Eig_Procedures .
!                                                                              
!                                                     
! Further Details
! ===============
!
!    The program also shows the use of subroutine TRID_INVITER in module Eig_Procedures.
!
! LATEST REVISION : 22/07/2010
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : lgl, i4b, stnd, true, false, zero, one, two, c50,     &
                         trid_inviter, symtrid_qri2, norm, unit_matrix 
#ifdef _MATMUL
    use Statpack, only : matmul=>matmul2
#endif
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=4000, neig=2
!   
    real(stnd), parameter  :: fudge=c50
!
    character(len=*), parameter :: name_proc='Example 2 of symtrid_qri2'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                              :: err1, err2, err, eps
    real(stnd), dimension(n)                :: d, e, e2, eigval
    real(stnd), dimension(n,neig)           :: eigvec
    real(stnd), allocatable, dimension(:,:) :: a, a2, resid
!
    integer      :: iok
    integer(i4b) :: maxiter=2, l
!
    logical(lgl) :: failure, do_test
!   
    character    :: sort='d'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = fudge*epsilon( err )
!
    err = zero
    do_test = true
!
!   GENERATE A (1-2-1) TRIDIAGONAL MATRIX.
!   THE DIAGONAL ELEMENTS ARE STORED IN d .
!   THE OFF-DIAGONAL ELEMENTS ARE STORED IN e .
!
    call random_number( d(:n) )
    call random_number( e(:n) )
!
!    d(:n) = two
!    e(:n) = one
!
!   SAVE THE TRIDIAGONAL FORM .
!
    eigval(:n) = d(:n)
    e2(:n)     = e(:n)
!
!   COMPUTE EIGENVALUES OF THE TRIDIAGONAL MATRIX.
!
    call symtrid_qri2( eigval(:n), e2(:n), failure, sort=sort )
!
    if ( .not. failure ) then
!
!       COMPUTE THE FIRST neig EIGENVECTORS OF THE (1-2-1) TRIDIAGONAL MATRIX BY maxiter INVERSE ITERATIONS.
!
        call trid_inviter( d(:n), e(:n), eigval(:neig), eigvec(:n,:neig), failure, maxiter=maxiter )
!
        if ( do_test ) then
!
            allocate( a(n,n), a2(neig,neig), resid(n,neig), stat=iok )
!
            if ( iok/=0 ) then
                write (prtunit,*)  'Problem in attempt to allocate memory !'
                stop
            end if
!
!           FORM THE TRIDIAGONAL MATRIX
!
            do l = 1_i4b, n-1_i4b
                a(l,l)       = d(l)
                a(l+1_i4b,l) = e(l)
                a(l,l+1_i4b) = e(l)
            end do
            a(n,n) = d(n)
!
!           CHECK FOR SMALL RESIDUALS OF THE EXPRESSION a*u - u*s
!           WHERE s ARE THE EIGENVALUES AND u THE EIGENVECTORS OF THE TRIDIAGONAL MATRIX a.
!
            resid = matmul( a, eigvec ) - eigvec*spread( eigval(:neig), 1, n )
            err1 = norm(resid)/( norm(a)*real(n,stnd) )
!
!           CHECK FOR SMALL RESIDUALS OF THE EXPRESSION I - u**(t)*u
!           WHERE u are THE EIGENVECTORS OF THE TRIDIAGONAL MATRIX a.
!
            call unit_matrix( a2 )
!
            resid(:neig,:neig) = a2 - matmul( transpose( eigvec ), eigvec )
            err2 = norm(resid(:neig,:neig))/real(n,stnd)
!
            err = max( err1, err2 )
!
            deallocate( a, a2, resid )
!
        end if
!
    end if
!
    if ( err<=eps .and. .not.failure ) then
        write (prtunit,*) 'Example 2 of SYMTRID_QRI2 is correct'
    else
        write (prtunit,*) 'Example 2 of SYMTRID_QRI2 is incorrect'
    end if
!
!
! END OF PROGRAM ex2_symtrid_qri2
! ===============================
!
end program ex2_symtrid_qri2
