module Derived_Types
! ####################################################################################
!
! Copyright 2018 IRD
!
! This file is part of statpack.
!
! statpack is free software: you can redistribute it and/or modify
! it under the terms of the GNU Lesser General Public License as
! published by the Free Software Foundation, either version 3 of 
! the License, or (at your option) any later version.
!
! statpack is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU Lesser General Public License for more details.
!
! You can find a copy of the GNU Lesser General Public License
! in the statpack/doc directory.
!
! ####################################################################################
!                                                                                    *
! ************************************************************************************
! MODULE EXPORTING DERIVED DATA TYPES FOR SPARSE REAL AND COMPLEX MATRICES OF KIND   *
! stnd AND extd.                                                                     * 
!                                                                                    * 
! THE AVAILABLE DERIVED DATA TYPES ARE DEFINED AS FOLLOW:                            * 
!                                                                                    * 
!    type sprs2_stnd                                                                 * 
!        integer(i4b) :: n, len                                                      * 
!        real(stnd),   dimension(:), pointer :: val                                  * 
!        integer(i4b), dimension(:), pointer :: irow                                 * 
!        integer(i4b), dimension(:), pointer :: jcol                                 * 
!    end type sprs2_stnd                                                             * 
!                                                                                    * 
!    type sprs2_extd                                                                 *
!        integer(i4b) :: n, len                                                      *
!        real(extd),   dimension(:), pointer :: val                                  *
!        integer(i4b), dimension(:), pointer :: irow                                 *
!        integer(i4b), dimension(:), pointer :: jcol                                 *
!    end type sprs2_extd                                                             *
!                                                                                    *
!    type sprs2_stndc                                                                *
!        integer(i4b) :: n, len                                                      *
!        complex(stnd), dimension(:), pointer :: val                                 *
!        integer(i4b),  dimension(:), pointer :: irow                                *
!        integer(i4b),  dimension(:), pointer :: jcol                                *
!    end type sprs2_stndc                                                            *
!                                                                                    *
!    type sprs2_extdc                                                                *
!        integer(i4b) :: n, len                                                      *
!        complex(extd), dimension(:), pointer :: val                                 *
!        integer(i4b),  dimension(:), pointer :: irow                                *
!        integer(i4b),  dimension(:), pointer :: jcol                                *
!    end type sprs2_extdc                                                            *
!                                                                                    * 
! LATEST REVISION : 21/03/2018                                                       *
!                                                                                    *
! ####################################################################################
!                                                                                    *
! ************************************************************************************
!
!
! USED MODULES
! ============
!   
    use Select_Parameters, only : i4b, stnd, extd
!
!
! PUBLIC ENTITIES 
! ===============
!
    private
    public :: sprs2_stnd, sprs2_stndc, sprs2_extd, sprs2_extdc
!
!
! DERIVED DATA TYPES FOR SPARSE MATRICES WITH KIND stnd AND extd 
! ==============================================================
!
    type sprs2_stnd
        integer(i4b) :: n, len
        real(stnd),   dimension(:), pointer :: val
        integer(i4b), dimension(:), pointer :: irow
        integer(i4b), dimension(:), pointer :: jcol
    end type sprs2_stnd
!
    type sprs2_extd
        integer(i4b) :: n, len
        real(extd),   dimension(:), pointer :: val
        integer(i4b), dimension(:), pointer :: irow
        integer(i4b), dimension(:), pointer :: jcol
    end type sprs2_extd
!
    type sprs2_stndc
        integer(i4b) :: n, len
        complex(stnd), dimension(:), pointer :: val
        integer(i4b),  dimension(:), pointer :: irow
        integer(i4b),  dimension(:), pointer :: jcol
    end type sprs2_stndc
!
    type sprs2_extdc
        integer(i4b) :: n, len
        complex(extd), dimension(:), pointer :: val
        integer(i4b),  dimension(:), pointer :: irow
        integer(i4b),  dimension(:), pointer :: jcol
    end type sprs2_extdc
!
!
! *****************************
! END OF MODULE Derived_Types *
! *****************************
!
end module Derived_Types
