program test2_bd_deflate
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine BD_DEFLATE in module SVD_Procedures.
!                                                                              
!                                                                              
! Further Details
! ===============
!                                                                              
!    The program assumes that subroutine BD_SINGVAL in module SVD_Procedures is correct.
!                                                                              
! LATEST REVISION : 02/10/2018
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : lgl, i4b, stnd, n1_def, n2_def, n3_def,     &
                         true, false, zero, one, c400, gen_bd_mat,   &
                         lamch, bd_singval, bd_deflate, norm
!
#ifdef _F2003
    use ieee_arithmetic, only : ieee_support_datatype, ieee_is_normal
#endif
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n1=n1_def, n2=n2_def, n3=n3_def, nsing_def=4
!   
    real(stnd), parameter :: fudge=c400
!
    character(len=*), parameter :: name_proc='bd_deflate'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                              :: err1, err2, err3, err, eps, eps0,  &
                                               eps2, ulp, safmin, abstol, norma
    real(stnd), allocatable, dimension(:)   :: d, e, singval
    real(stnd), allocatable, dimension(:,:) :: a, leftvec, rightvec, res
!
    integer(i4b)     :: i, j, k, l, m, n, nsing, success, notdone, max_qr_steps=10
    integer          :: iok
!
    logical(lgl)     :: failure1, failure2, upper, ortho, details
!   
    character        :: sort='d'
!   
!   
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
#ifdef _TEST_DETAILS
    details = .true.
#else
    details = .false.
#endif
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    ulp  = epsilon( err )
    eps  = fudge*ulp
    eps0 = sqrt( ulp )
!    eps0 = eps
!
    safmin = lamch('s')
    abstol = sqrt( safmin )
!
    upper = true
!
    notdone = 0_i4b
    success = 0_i4b
    m       = 0_i4b
!
    do n = n1, n2, n3
!
!       ALLOCATE WORK ARRAYS.
!
        allocate( d(n), e(n), singval(n), leftvec(n,n), rightvec(n,n),    &
                  a(n,n), res(n,n), stat=iok   )
!
        if ( iok/=0 ) then
!
            if ( allocated(d) )        deallocate( d )
            if ( allocated(e) )        deallocate( e )
            if ( allocated(singval) )  deallocate( singval )
            if ( allocated(leftvec) )  deallocate( leftvec )
            if ( allocated(rightvec) ) deallocate( rightvec )
            if ( allocated(a) )        deallocate( a )
            if ( allocated(res) )      deallocate( res )
!
            cycle
!
        end if
!
        do i = 1_i4b, 57_i4b
!
!           SKIP GRADED MATRICES WITH A GEOMETRIC PROGRESSION TO AVOID
!           OVERFLOW PROBLEMS IN SINGLE PRECISION.
!
            if ( (k==29_i4b .or. k==31_i4b .or. k==32_i4b .or. k==33_i4b) .and. (n>60_i4b) ) cycle
!
!           GENERATE A BIDIAGONAL MATRIX.
!           THE DIAGONAL ELEMENTS ARE STORED IN d.
!           THE OFF-DIAGONAL ELEMENTS ARE STORED IN e.
!
            call gen_bd_mat( i, d(:n), e(:n) )
!
#ifdef _F2003
            if ( ieee_support_datatype( d ) ) then
                if ( .not.all( ieee_is_normal( d(:n) ) ) ) cycle
                if ( .not.all( ieee_is_normal( e(:n) ) ) ) cycle
            end if
#endif
!
!           COMPUTE ALL SINGULAR VALUES OF THE BIDIAGONAL MATRIX BY A BISECTION METHOD WITH HIGH ACCURACY.
!           THE SINGULAR VALUES ARE COMPUTED WITH MAXIMUM ACCURACY WHEN OPTIONAL PARAMETER abstol IS SET TO
!           sqrt( lamch('s') ), WHICH IS THE SQUARE ROOT OF THE UNDERFLOW THRESHOLD.
!
            call bd_singval( d(:n), e(:n), l, singval(:n), failure1, sort=sort, ls=n, abstol=abstol )
!
            if ( failure1 ) cycle
!
!           COMPUTE THE NORM OF THE BIDIAGONAL MATRIX.
!
            norma = norm( singval(:n) )
!
!           COMPUTE SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
            do k = 1_i4b, 2_i4b
!
                upper = .not.upper
!
                if ( k==1_i4b ) then
!
                    nsing = min( n, nsing_def )
!
                else
!
                    nsing = n
!
                end if
!
                do j = 1_i4b, 2_i4b
!
                    if ( j==1_i4b ) then
!
                        eps2 = eps
!
                        ortho = true
!
                    else
!
                        eps2 = eps0
!
                        ortho = false
!
                    end if
!
                    call bd_deflate( upper, d(:n), e(:n), singval(:nsing),            &
                                     leftvec(:n,:nsing), rightvec(:n,:nsing),         &
                                     failure2, ortho=ortho, max_qr_steps=max_qr_steps )
!
                    if ( failure2 ) then
!
                        notdone = notdone + 1_i4b
!
                        cycle
!
                    end if
!
                    m = m + 1_i4b
!
!                   FORM THE BIDIAGONAL MATRIX AND
!                   TEST ACCURACY OF THE RESULTS.
!
                    a(:n,:n) = zero
!
                    if ( upper ) then
!
                        do l = 1_i4b, n-1_i4b
                            a(l,l)       = d(l)
                            a(l,l+1_i4b) = e(l+1_i4b)
                        end do
!
                    else
!
                        do l = 1_i4b, n-1_i4b
                            a(l,l)       = d(l)
                            a(l+1_i4b,l) = e(l+1_i4b)
                        end do
!
                    end if
!
                    a(n,n) = d(n)
!
!                   CHECK FOR SMALL RESIDUALS OF THE EXPRESSION a*v - u*s
!                   WHERE s ARE THE SINGULAR VALUES, u THE LEFT SINGULAR VECTORS AND
!                   v THE RIGHT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
                    res(:n,:nsing) = matmul( a(:n,:n), rightvec(:n,:nsing) )    &
                                     - leftvec(:n,:nsing)*spread(singval(:nsing),1,n)
!
                    if ( norma/=zero ) then
                        err1 = maxval( norm(res(:n,:nsing), dim=2_i4b) )/( norma*real(n,stnd) )
                    else
                        err1 = maxval( norm(res(:n,:nsing), dim=2_i4b) )/real(n,stnd)
                    end if
!
!                   CHECK FOR SMALL RESIDUALS OF THE EXPRESSION I - u**(t)*u
!                   WHERE u are LEFT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
                    res(:nsing,:nsing) = matmul( transpose( leftvec(:n,:nsing) ), leftvec(:n,:nsing) )
!
                    do l = 1_i4b, nsing
                        res(l,l) = one - res(l,l)
                    end do
!
                    err2 = maxval( norm( res(:nsing,:nsing), dim=2_i4b) )/real(n,stnd)
!
!                   CHECK FOR SMALL RESIDUALS OF THE EXPRESSION I - v**(t)*v
!                   WHERE v are RIGHT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
                    res(:nsing,:nsing) = matmul( transpose( rightvec(:n,:nsing) ), rightvec(:n,:nsing) )
!
                    do l = 1_i4b, nsing
                        res(l,l) = one - res(l,l)
                    end do
!
                    err3 = maxval( norm( res(:nsing,:nsing), dim=2_i4b) )/real(n,stnd)
!
                    if ( err1<=eps .and. max(err2,err3)<=eps2 ) then
!
                        success = success + 1_i4b
!
                        if ( details ) then
!
                            write (prtunit,*) 'Test ',m,' is correct'
!
                            write (prtunit,*) 'n= ', n,' , nsing= ', nsing,' , case= ', i,   &
                                              ' , upper= ', upper,' , ortho= ', ortho
!
!                           CHECK FOR SMALL RESIDUALS OF THE EXPRESSION a*v - u*s
!                           WHERE s ARE THE SINGULAR VALUES, u THE LEFT SINGULAR VECTORS AND
!                           v THE RIGHT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
                            write (prtunit,*) 'Accuracy of the computed singular triplets             = ', &
                                              err1/ulp, ' (must be less than :', fudge, ' )'
!
!                           CHECK FOR SMALL RESIDUALS OF THE EXPRESSION I - u**(t)*u
!                           WHERE u are LEFT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
                            write (prtunit,*) 'Orthogonality of the computed left singular vectors    = ', &
                                              err2/ulp, ' (must be less than :', fudge, ' )'
!
!                           CHECK FOR SMALL RESIDUALS OF THE EXPRESSION I - v**(t)*v
!                           WHERE v are RIGHT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
                            write (prtunit,*) 'Orthogonality of the computed right singular vectors   = ', &
                                              err3/ulp, ' (must be less than :', fudge, ' )'
!
                            write (prtunit,*)
!
                        end if
!
                    else
!
                        write (prtunit,*) 'Test ',m,' is incorrect'
!
                        if ( j==1_i4b ) then
!
                            err = max( err1, err2, err3 )
                        
!
                            write (prtunit,*) '=> ',err,' > ', eps,' , n= ', n,' , nsing= ', nsing,    &
                                              ' , case= ', i,' , upper= ', upper,' , ortho= ', ortho,  &
                                              ' , failure= ', failure2
!
                        else
!
                            err = max( err2, err3 )
!
                            write (prtunit,*) '=> ',err1,' > ', eps,' or ', err,' > ',eps2,   &
                                              ' , n= ', n,' , nsing= ', nsing,' , case= ', i, &
                                              ' , upper= ', upper,' , ortho= ', ortho,        &
                                              ' , failure= ', failure2
!
                        end if
!
!                       CHECK FOR SMALL RESIDUALS OF THE EXPRESSION a*v - u*s
!                       WHERE s ARE THE SINGULAR VALUES, u THE LEFT SINGULAR VECTORS AND
!                       v THE RIGHT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
                        write (prtunit,*) 'Accuracy of the computed singular triplets             = ', &
                                          err1/ulp, ' (must be less than :', fudge, ' )'
!
!                       CHECK FOR SMALL RESIDUALS OF THE EXPRESSION I - u**(t)*u
!                       WHERE u are LEFT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
                        write (prtunit,*) 'Orthogonality of the computed left singular vectors    = ', &
                                          err2/ulp, ' (must be less than :', fudge, ' )'
!
!                       CHECK FOR SMALL RESIDUALS OF THE EXPRESSION I - v**(t)*v
!                       WHERE v are RIGHT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
                        write (prtunit,*) 'Orthogonality of the computed right singular vectors   = ', &
                                          err3/ulp, ' (must be less than :', fudge, ' )'
!
                        write (prtunit,*)
!
                    end if
!
                end do
!
            end do
!
        end do
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( d, e, a, res, singval, leftvec, rightvec )
!
    end do
!
    if ( notdone/=0_i4b ) then
        write (prtunit,*) notdone, 'tests not done'
    end if
!
    if ( m/=success ) then
        write (prtunit,*) m - success, 'tests are incorrect'
    end if
!
    write (prtunit,*) success, 'tests are correct'
!
!
! END OF PROGRAM test2_bd_deflate
! ===============================
!
end program test2_bd_deflate
