program test2_bd_svd2
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine BD_SVD2 in module SVD_Procedures.
!                                                          
!                                                         
!
! LATEST REVISION : 31/07/2018
!                                                          
! ================================================================================================
!
!
! USED MODULES
! ============
!
    use Statpack, only : lgl, i4b, stnd, n1_def, n2_def, n3_def,   &
                         true, zero, one, c50, norm,               &
                         bd_svd2, gen_bd_mat, unit_matrix
!   
#ifdef _F2003
    use ieee_arithmetic, only : ieee_support_datatype, ieee_is_normal
#endif
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n1=n1_def, n2=n2_def, n3=n3_def
!
    character(len=*), parameter :: name_proc='bd_svd2'
!   
    real(stnd), parameter  :: fudge=c50
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                              :: err, err1, err2, err3, err4, eps, &
                                               ulp, norma
    real(stnd), allocatable, dimension(:)   :: d, e, e2, singval
    real(stnd), allocatable, dimension(:,:) :: a, leftvec, rightvec, res
!
    integer(i4b) :: k, l, m, n, o, success, notdone
    integer      :: iok
!
    logical(lgl) :: failure, failure2, upper, perfect_shift, details
!   
    character    :: sort='d'
!   
!   
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
#ifdef _TEST_DETAILS
    details = .true.
#else
    details = .false.
#endif
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    ulp    = epsilon( err )
    eps    = fudge*ulp
!
    upper         = true
    perfect_shift = true
!
    notdone = 0_i4b
    success = 0_i4b
    m       = 0_i4b
!
    do n = n1, n2, n3
!
!       ALLOCATE WORK ARRAYS.
!
        allocate( d(n), e(n), e2(n), singval(n), leftvec(n,n),  &
                  rightvec(n,n), a(n,n), res(n,n), stat=iok     )
!
        if ( iok/=0 ) then
!
            if ( allocated(d) )        deallocate( d )
            if ( allocated(e) )        deallocate( e )
            if ( allocated(e2) )       deallocate( e2 )
            if ( allocated(singval) )  deallocate( singval )
            if ( allocated(leftvec) )  deallocate( leftvec )
            if ( allocated(rightvec) ) deallocate( rightvec )
            if ( allocated(a) )        deallocate( a )
            if ( allocated(res) )      deallocate( res )
!
            cycle
!
        end if
!
        do o = 1_i4b, 2_i4b
!
            upper         = .not.upper
            perfect_shift = .not.perfect_shift
!
            do k = 1_i4b, 57_i4b
!
!               SKIP SOME GRADED MATRICES WITH A GEOMETRIC PROGRESSION TO AVOID
!               OVERFLOW PROBLEMS IN SINGLE PRECISION.
!
                if ( (k==29_i4b .or. k==31_i4b .or. k==32_i4b .or. k==33_i4b) .and. (n>60_i4b) ) cycle
!
!               GENERATE A BIDIAGONAL MATRIX.
!               THE DIAGONAL ELEMENTS ARE STORED IN d.
!               THE OFF-DIAGONAL ELEMENTS ARE STORED IN e.
!
                call gen_bd_mat( k, d(:n), e(:n) )
!
#ifdef _F2003
                if ( ieee_support_datatype( d ) ) then
                    if ( .not.all( ieee_is_normal( d(:n) ) ) ) cycle
                    if ( .not.all( ieee_is_normal( e(:n) ) ) ) cycle
                end if
#endif
!
!               SAVE THE BIDIAGONAL MATRIX FOR LATER USE.
!
                singval(:n) = d(:n)
                e2(:n)      = e(:n)
!
!               COMPUTE SINGULAR VALUES AND SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
                call unit_matrix( leftvec  )
                call unit_matrix( rightvec )
!
                call bd_svd2( upper, singval(:n), e2(:n), failure,  &
                             leftvec(:n,:n), rightvec(:n,:n),       &
                             sort=sort, perfect_shift=perfect_shift )
!
                if ( failure ) then
!
                    notdone = notdone + 1_i4b
!
                    cycle
!
                end if
!
!               COMPUTE THE NORM OF THE BIDIAGONAL MATRIX.
!
                norma = norm( singval(:n) )
!
                m = m + 1_i4b
!
!               NOW, TEST ACCURACY OF THE RESULTS.
!
                a(:n,:n) = zero
!
                if ( upper ) then
!
                    do l = 1_i4b, n-1_i4b
                        a(l,l)       = d(l)
                        a(l,l+1_i4b) = e(l+1_i4b)
                    end do
!
                else
!
                    do l = 1_i4b, n-1_i4b
                        a(l,l)       = d(l)
                        a(l+1_i4b,l) = e(l+1_i4b)
                    end do
!
                end if
!
                a(n,n) = d(n)
!
!               CHECK FOR SMALL RESIDUALS OF THE EXPRESSION a*v - u*s
!               WHERE s ARE THE SINGULAR VALUES, u THE LEFT SINGULAR VECTORS AND
!               v THE RIGHT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
                res(:n,:n) = matmul( a(:n,:n), transpose( rightvec(:n,:n) ) )  &
                             - leftvec(:n,:n)*spread(singval(:n),1,n)
!
                if ( norma/=zero ) then
                    err1 = maxval( norm(res(:n,:n), dim=2_i4b) )/( norma*real(n,stnd) )
                else
                    err1 = maxval( norm(res(:n,:n), dim=2_i4b) )/real(n,stnd)
                end if
!
!               CHECK FOR SMALL RESIDUALS OF THE EXPRESSION I - u**(t)*u
!               WHERE u are LEFT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
                res(:n,:n) = matmul( transpose( leftvec(:n,:n) ), leftvec(:n,:n) )
!
                do l = 1_i4b, n
                    res(l,l) = one - res(l,l)
                end do
!
                err2 = maxval( norm(res(:n,:n), dim=2_i4b) )/real(n,stnd)
!
!               CHECK FOR SMALL RESIDUALS OF THE EXPRESSION I - v**(t)*v
!               WHERE v are RIGHT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
                res(:n,:n) = matmul( rightvec(:n,:n), transpose( rightvec(:n,:n) ) )
!
                do l = 1_i4b, n
                    res(l,l) = one - res(l,l)
                end do
!
                err3 = maxval( norm(res(:n,:n), dim=2_i4b) )/real(n,stnd)
!
!               RECOMPUTE SINGULAR VALUES OF THE BIDIAGONAL MATRIX.
!
                singval(:n) = d(:n)
                e2(:n)      = e(:n)
!
                call bd_svd2( upper, singval(:n), e2(:n), failure2, sort=sort )
!
                if ( .not.failure2 ) then
!
!                   CHECK AGAIN FOR SMALL RESIDUALS OF THE EXPRESSION a*v - u*s
!                   WHERE s ARE THE SINGULAR VALUES, u THE LEFT SINGULAR VECTORS AND
!                   v THE RIGHT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
                    res(:n,:n) = matmul( a(:n,:n), transpose( rightvec(:n,:n) ) )    &
                                 - leftvec(:n,:n)*spread(singval(:n),1,n)
!
                    if ( norma/=zero ) then
                        err4 = maxval( norm(res(:n,:n), dim=2_i4b) )/( norma*real(n,stnd) )
                    else
                        err4 = maxval( norm(res(:n,:n), dim=2_i4b) )/real(n,stnd)
                    end if
!
                else
!
                    err4 = zero
!
                end if
!
                err = max( err1, err2, err3, err4 )
!
                if ( err<=eps  .and. .not.failure2 ) then
!
                    success = success + 1_i4b
!
                    if ( details ) then
!
                        write (prtunit,*) 'Test ',m,' is correct'
                        write (prtunit,*) 'n= ', n,' , case= ', k,' , upper= ', upper,   &
                                          ' , perfect_shift= ', perfect_shift
!
!                       CHECK FOR SMALL RESIDUALS OF THE EXPRESSION a*v - u*s
!                       WHERE s ARE THE SINGULAR VALUES, u THE LEFT SINGULAR VECTORS AND
!                       v THE RIGHT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
                        write (prtunit,*) 'Accuracy of the computed singular triplets             = ', &
                                          max(err1,err4)/ulp, ' (must be less than :', fudge, ' )'
!
!                       CHECK FOR SMALL RESIDUALS OF THE EXPRESSION I - u**(t)*u
!                       WHERE u are LEFT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
                        write (prtunit,*) 'Orthogonality of the computed left singular vectors    = ', &
                                          err2/ulp, ' (must be less than :', fudge, ' )'
!
!                       CHECK FOR SMALL RESIDUALS OF THE EXPRESSION I - v**(t)*v
!                       WHERE v are RIGHT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
                        write (prtunit,*) 'Orthogonality of the computed right singular vectors   = ', &
                                          err3/ulp, ' (must be less than :', fudge, ' )'
!
                        write (prtunit,*)
!
                    end if
!
                else
!
                    write (prtunit,*) 'Test ',m,' is incorrect'
                    write (prtunit,*) '=> ',err,' > ',eps,' , n= ', n,' , case= ', k,          &
                                      ' , upper= ', upper,' , perfect_shift= ', perfect_shift, &
                                      ' , failure= ', failure, ' , failure2= ', failure2
!
!                   CHECK FOR SMALL RESIDUALS OF THE EXPRESSION a*v - u*s
!                   WHERE s ARE THE SINGULAR VALUES, u THE LEFT SINGULAR VECTORS AND
!                   v THE RIGHT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
                    write (prtunit,*) 'Accuracy of the computed singular triplets             = ', &
                                      max(err1,err4)/ulp, ' (must be less than :', fudge, ' )'
!
!                   CHECK FOR SMALL RESIDUALS OF THE EXPRESSION I - u**(t)*u
!                   WHERE u are LEFT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
                    write (prtunit,*) 'Orthogonality of the computed left singular vectors    = ', &
                                       err2/ulp, ' (must be less than :', fudge, ' )'
!
!                   CHECK FOR SMALL RESIDUALS OF THE EXPRESSION I - v**(t)*v
!                   WHERE v are RIGHT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX.
!
                    write (prtunit,*) 'Orthogonality of the computed right singular vectors   = ', &
                                      err3/ulp, ' (must be less than :', fudge, ' )'
!
                    write (prtunit,*)
!
                end if
!
            end do
!
        end do
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( d, e, e2, singval, a, res, leftvec, rightvec )
!
    end do
!
    if ( notdone/=0_i4b ) then
        write (prtunit,*) notdone, 'tests not done'
    end if
!
    if ( m/=success ) then
        write (prtunit,*) m - success, 'tests are incorrect'
    end if
!
    write (prtunit,*) success, 'tests are correct'
!
!
! END OF PROGRAM test2_bd_svd2
! ============================
!
end program test2_bd_svd2
