program test2_trid_cmp2
!
!
! Purpose
! =======
!
!   This program is intended to test subroutineS TRID_CMP2 and TRID_SOLVE in
!   module EIG_Procedures.
!                                                                              
!
! LATEST REVISION : 21/05/2018
!                                                                              
! ================================================================================================
!
!
! USED MODULES
! ============
!
    use Statpack, only : lgl, i4b, stnd, n1_def, n2_def, n3_def,        &
                         zero, half, c100, allocate_error,              &
                         merror, trid_cmp2, trid_solve, sym_trid_solve, &
                         norm, lamch
!
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!
!
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n1=n1_def, n2=n2_def, n3=n3_def, neig=10
!
    character(len=*), parameter :: name_proc='trid_cmp2'
!   
    real(stnd), parameter  :: fudge=c100
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                                :: err, eps, eps2, ulp, safmin, normx,    &
                                                 tmp, scale
    real(stnd), allocatable, dimension(:)     :: d, e, b, b2, x, res, eigval
    real(stnd), allocatable, dimension(:,:)   :: a, sub, diag, sup1, sup2
!
    integer(i4b)     :: i, l, m, n, success, notdone
    integer          :: iok
!
    logical(lgl)                              :: details
    logical(lgl), allocatable, dimension(:,:) :: perm
!
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
#ifdef _TEST_DETAILS
    details = .true.
#else
    details = .false.
#endif
!
!   ALLOCATE WORK ARRAY.
!
    allocate( eigval(neig), stat=iok )
!
    if ( iok/=0 ) then
        call merror( name_proc//allocate_error )
    end if
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    ulp    = epsilon( err )
    eps    = fudge*ulp
    safmin = lamch( 'S' )
!
    notdone = 0_i4b
    success = 0_i4b
    m       = 0_i4b
!
    do n = n1, n2, n3
!
!       ALLOCATE WORK ARRAYS.
!
        allocate( a(n,n), d(n), e(n), sub(neig,n), diag(neig,n),  &
                  sup1(neig,n), sup2(neig,n), perm(neig,n),       &
                  b(n), b2(n), x(n), res(n), stat=iok )
!
        a(:n,:n) = zero
!
        if ( iok/=0 ) cycle
!
!       GENERATE A RANDOM TRIDIAGONAL MATRIX .
!       THE DIAGONAL ELEMENTS ARE STORED IN d.
!       THE OFF-DIAGONAL ELEMENTS ARE STORED IN e.
!
        call random_number( d(:n) )
!
        d(:n) = d(:n) - half
!
        call random_number( tmp )
!
        if ( tmp>safmin ) then
            d(:n) = d(:n)/tmp
        end if
!
        call random_number( e(:n) )
!
        e(:n) = e(:n) - half
!
        call random_number( tmp )
!
        if ( tmp>safmin ) then
            e(:n) = e(:n)/tmp
        end if
!
!       GENERATE neig RANDOM SHIFTS.
!
        call random_number( eigval(:neig) )
!
        if ( tmp>safmin ) then
            eigval(:neig) = eigval(:neig)/tmp
        end if
!
!       GENERATE A RANDOM SOLUTION VECTOR x.
!
        call random_number( x(:n) )
!
        call random_number( tmp )
!
        if ( tmp>safmin ) then
            x(:n) = x(:n)/tmp
        end if
!
        normx = norm( x(:n) )
!
!       FACTORIZE THE TRIDIAGONAL MATRIX.
!
        call trid_cmp2( d(:n), e(:n), eigval(:neig), sub, diag, sup1, sup2, perm )
!
        eps2 = real( n, stnd)*eps
!
        do i = 1_i4b, neig
!
!           FORM THE TRIDIAGONAL MATRIX.
!
            do l = 1_i4b, n-1_i4b
                a(l,l)       = d(l) - eigval(i)
                a(l+1_i4b,l) = e(l)
                a(l,l+1_i4b) = e(l)
            end do
!
            a(n,n) = d(n) - eigval(i)
!
!           COMPUTE THE RIGHT HAND SIDE VECTOR b.
!
            b(:n) = matmul( a, x )
!
            b2(:n) = b(:n)
!
!           SOLVE THE TRIDIAGONAL SYSTEM.
!
            call sym_trid_solve( sub(i,:n), diag(i,:n), sup1(i,:n),   &
                                 sup2(i,:n), perm(i,:n), b2, scale    )
!
            if ( scale==zero ) then
!
                notdone = notdone + 1_i4b
!
                cycle
!
            end if
!
!           RESOLVE THE TRIDIAGONAL SYSTEM FOR TESTING trid_solve SUBROUTINE.
!
            call trid_solve( sub(i,:n), diag(i,:n), sup1(i,:n),   &
                             sup2(i,:n), perm(i,:n), b            )
!
            m = m + 1_i4b
!
!           CHECK THE RESULTS FOR SMALL RESIDUALS.
!
            res(:n) = x(:n) - b(:n)/scale
!
            err = norm( res(:n) )/normx
!
            if ( err<=eps2 ) then
!
                success = success + 1_i4b
!
                if ( details ) then
!
                    write (prtunit,*) 'Test ',m,' is correct'
!
                    write (prtunit,*) 'n= ', n, ' , shift= ', eigval(i)
!
!                   PRINT RELATIVE ERROR OF SOLUTION.
!
                    write (prtunit,*) 'Relative error of the computed solution = ', err
!
                    write (prtunit,*)
!
                end if
!
            else
!
                write (prtunit,*) 'Test ',m,' is incorrect'
!
                write (prtunit,*) '=> ',err,' > ',eps2,' , n= ', n, ' , shift= ', eigval(i)
!
!               PRINT RELATIVE ERROR OF SOLUTION.
!
                write (prtunit,*) 'Relative error of the computed solution = ', err
!
                write (prtunit,*)
!
            end if
!
        end do
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( a, d, e, sub, diag, sup1, sup2,   &
                    perm, b, b2, x, res  )
!
    end do
!
!   DEALLOCATE WORK ARRAY.
!
    deallocate( eigval )
!
    if ( notdone/=0_i4b ) then
        write (prtunit,*) notdone, 'tests not done'
    end if
!
    if ( m/=success ) then
        write (prtunit,*) m - success, 'tests are incorrect'
    end if
!
    write (prtunit,*) success, 'tests are correct'
!
!
! END OF PROGRAM test2_trid_cmp2
! ==============================
!
end program test2_trid_cmp2
