program test_bd_coef2
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine BD_COEF2 in module Time_Series_Procedures.
!                                                                              
!                                                                              
! Further Details
! ===============
!
!    The program assumes that subroutine FREQ_FUNC in module Time_Series_Procedures is correct.
!                                                                              
! LATEST REVISION : 20/04/2010
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, half, one, bd_coef2, freq_func
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT .
!
    integer(i4b), parameter :: prtunit=6, n=2000
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                            :: diff, fch, fcl, win, tmp
    real(stnd), dimension(n)              :: freqr, freq
    real(stnd), dimension(:), allocatable :: coef
!
    integer(i4b) :: i, k, k1, k2, pl, ph, nmask, nmask2
!   
    integer :: iok
!   
    logical(lgl), dimension(n) :: mask, mask2
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='bd_coef2'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TESTS.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   pl IS THE MINIMUM PERIOD OF OSCILLATION OF DESIRED COMPONENT.
!   ph IS THE MAXIMUM PERIOD OF OSCILLATION OF DESIRED COMPONENT.
!   pl AND ph ARE EXPRESSED IN NUMBER OF OBSERVATIONS, I.E. pl=6(18) and ph=32(96) SELECT
!   PERIODS BETWEEN 1.5 YRS AND 8 YRS FOR QUARTERLY (MONTHLY) DATA.
!
    pl  = 35_i4b
    ph  = 192_i4b
!
!   COMPUTE THE CORRESPONDING CUTOFF FREQUENCIES.
!
    fcl  = one/real( pl, stnd )
    fch  = one/real( ph, stnd )
!
!   NOW SELECT THE NUMBER OF FILTER COEFFICIENTS k FOR THE FILTER.
!
    k1  = ceiling( one/(half-fcl) )
    k2  = ceiling(  2.6_stnd/(fcl-fch) )
    k   = max( k1, k2, ph+1_i4b )
!
!   CHECK IF k IS ODD.
!
    if ( (k/2_i4b)*2_i4b==k )  k = k + 1_i4b
!
!   ALLOCATE WORK ARRAY FOR THE FILTER COEFFICIENTS.
!
    allocate( coef(k), stat=iok )
!
    if ( iok==0 ) then
!
        do i = 1_i4b, 3_i4b
!
            select case( i )
!
                case( 1_i4b )
!
!                   HANNING FILTER.
!
                    win = half
!
                case( 2_i4b )
!
!                   HAMMING FILTER.
!
                    win = 0.54_stnd
!
                case( 3_i4b )
!
!                   IDEAL FILTER.
!
                    win = one
!
            end select
!
!           FUNCTION bd_coef2 COMPUTES THE K-TERM LEAST SQUARES APPROXIMATION TO AN
!           -IDEAL- BAND PASS FILTER WITH CUTOFF PERIODS PL AND PH (EG CUTOFF FREQUENCIES 1/PL AND 1/PH).
!
            coef(:k) = bd_coef2( pl, ph, k, win=win )
!
!           NOW COMPUTE THE TRANSFERT FUNCTION freqr(:) OF THE FILTER.
!
            call freq_func( n, coef(:k), freqr(:n), freq=freq(:n) )
!
            tmp = one/real( k, stnd )
!
            mask(:n) = freq(:n)>=( fch + tmp ) .and. freq(:n)<=( fcl - tmp )
            nmask    = count( mask(:n) )
!
            mask2(:n) = freq(:n)<( fch - tmp ) .or. freq(:n)>( fcl + tmp )
            nmask2    = count( mask2(:n) )
!
            if ( nmask/=0_i4b .and. nmask2/=0_i4b ) then
!
!               CHECK THE PASS-BAND AND THE STOP-BAND OF THE FILTER.
!
                diff = sum( freqr(:n), mask=mask(:n))/real( nmask, stnd )           &
                       - sum( freqr(:n), mask=mask2(:n) )/real( nmask2, stnd )  
!
            else
!
                diff = -one
!
            end if
!
!           TEST THE ACCURACY OF THE RESULTS.
!
            if ( diff>0.95_stnd  ) then
                write (prtunit,*) 'Test ',i,' is correct'
            else
                write (prtunit,*) 'Test ',i,' is incorrect'
                write (prtunit,*) '=> ',diff,' <= ',0.95_stnd
            end if
!
        end do
!
    else
!
        write (prtunit,*) 'Problem in attempt to allocate memory for tests 1, 2 and 3'
!
    end if
!
!
! END OF PROGRAM test_bd_coef2
! ============================
!
end program test_bd_coef2
