program test_comp_cor
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine comp_cor in module Mul_Stat_Procedures .
!                                                                              
! LATEST REVISION : 20/04/2010
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, machtiny, comp_cor
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=26, m=20, p=50, q=5
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                      :: err_xstat, err_ystat, err_cor, err, xyn, eps
    real(stnd), dimension(n,m,q)    :: xycor, xycor2
    real(stnd), dimension(q,2)      :: ystat, ystat2
    real(stnd), dimension(n,m,2)    :: xstat, xstat2
    real(stnd), dimension(n,m,p)    :: x, xt
    real(stnd), dimension(q,p)      :: y, yt
!
    integer(i4b) :: i, j, k, l, k2, inc
!
    logical(lgl) :: first, last
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='comp_cor'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TESTS.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = sqrt( epsilon(eps) )
!
!   GENERATE A RANDOM OBSERVATION ARRAY x .
!
    call random_number( x )
!
!   COMPUTE THE MEANS, VARIANCES OF x  .
!
    xstat(:,:,1_i4b) = sum( x, dim=3_i4b )/p
    xt = x - spread( xstat(:,:,1_i4b), dim=3_i4b, ncopies=p )
    xstat(:,:,2_i4b) = sum( xt**2, dim=3_i4b )
!
!   GENERATE A RANDOM OBSERVATION ARRAY y .
!
    call random_number( y )
!
!   COMPUTE THE MEANS, VARIANCES OF y  .
!
    ystat(:,1_i4b) = sum( y, dim=2_i4b )/p
    yt = y - spread( ystat(:,1_i4b), dim=2_i4b, ncopies=p )
    ystat(:,2_i4b) = sum( yt**2, dim=2_i4b )
!
!   COMPUTE THE CORRELATIONS BETWEEN x AND y  .
!
!    do k = 1_i4b, p
!        xycor(:n,:m) = xycor(:n,:m) + yt(k)*xt(:n,:m,k)
!    end do
!    xycor = xycor/( sqrt( ystat(2_i4b)*xstat(:n,:m,2_i4b) ) + machtiny )
!
    do k = 1_i4b, q
        xycor(:n,:m,k) = sum(spread(spread(yt(k,:),1,n),2,m)*xt,dim=3_i4b)/  &
                         ( sqrt( ystat(k,2_i4b)*xstat(:n,:m,2_i4b) ) + machtiny )
    end do
!
    xstat(:,:,2_i4b) = xstat(:,:,2_i4b)/p
    ystat(:,2_i4b) = ystat(:,2_i4b)/p
!
    do i = 1_i4b, 8_i4b
!
        inc = merge( 1_i4b, p, i<=4_i4b )
!
        select case( i )
!
            case( 1_i4b, 5_i4b )
!
                xstat2 = xstat
                ystat2 = ystat
                xycor2 = xycor
!
!               TESTS 1 AND 5 :
!
!               COMPUTE THE STATISTICS WITH comp_cor_rt .
!
                k2 = 0_i4b
                do k = 1_i4b, p, inc
!
                    k2 = min( k2+inc, p )
!
                    first = k==1_i4b
                    last = k2==p
!
                    call comp_cor( x(:n,:m,k:k2), y(1_i4b,k:k2), first, last,         &
                                   xstat2, ystat2(1_i4b,:), xycor2(:,:,1_i4b), xyn )
                end do
!
            case( 2_i4b, 6_i4b )
!
!               TESTS 2 AND 6 :
!
!
!               COMPUTE THE STATISTICS OF WITH comp_cor_rm .
!
                do j = 1_i4b, m
                    k2 = 0_i4b
                    do k = 1_i4b, p, inc
!
                        k2 = min( k2+inc, p )
!
                        first = k==1_i4b
                        last = k2==p
!
                        call comp_cor( x(:,j,k:k2), y(1_i4b,k:k2), first, last,                 &
                                       xstat2(:,j,:), ystat2(1_i4b,:), xycor2(:,j,1_i4b), xyn )
                    end do
                end do
!
            case( 3_i4b, 7_i4b )
!
!               TESTS 3 AND 7 :
!
!
!               COMPUTE THE STATISTICS WITH comp_cor_rv .
!
                do l = 1_i4b, n
                    do j = 1_i4b, m
                        k2 = 0_i4b
                        do k = 1_i4b, p, inc
!
                            k2 = min( k2+inc, p )
!
                            first = k==1_i4b
                            last = k2==p
!
                            call comp_cor( x(l,j,k:k2), y(1_i4b,k:k2), first, last,                  &
                                           xstat2(l,j,:), ystat2(1_i4b,:), xycor2(l,j,1_i4b), xyn )
                        end do
                    end do
                end do
!
            case( 4_i4b, 8_i4b )
!
!               TESTS 4 AND 8 :
!
!
!               COMPUTE THE STATISTICS WITH comp_cor_rm_rm .
!
                do j = 1_i4b, m
                    k2 = 0_i4b
                    do k = 1_i4b, p, inc
!
                        k2 = min( k2+inc, p )
!
                        first = k==1_i4b
                        last = k2==p
!
                        call comp_cor( x(:,j,k:k2), y(:,k:k2), first, last,                 &
                                       xstat2(:,j,:), ystat2(:,:), xycor2(:,j,:), xyn )
                    end do
                end do
!
        end select
!
!       CHECK THAT THE TWO SETS OF STATISTICS AGREE.
!
        err_xstat = maxval( abs( (xstat-xstat2)/xstat2 ) )
        err_ystat = maxval( abs( (ystat-ystat2)/ystat2 ) )
        err_cor  = maxval( abs( xycor-xycor2 ) )
!
!       PRINT THE RESULTS OF THE TESTS.
!
        err = max( err_xstat, err_ystat, err_cor )
!
        if ( err<=eps ) then
            write (prtunit,*) 'Test ',i,' is correct'
        else
            write (prtunit,*) 'Test ',i,' is incorrect'
            write (prtunit,*) '=> ',err,' > ',eps
        end if
!
    end do
!
!
! END OF PROGRAM test_comp_cor
! ============================
!
end program test_comp_cor
