program test_comp_cor_miss
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine comp_cor_miss in module Mul_Stat_Procedures .
!                                                                              
! LATEST REVISION : 20/04/2010
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, zero, one, comp_cor_miss
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=26, m=20, p=50, q=5
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                      :: err_xstat, err_ystat, err_cor, err, eps, miss=-999.99_stnd
    real(stnd), dimension(n,m,q,2)  :: xycor
    real(stnd), dimension(n,m,q,4)  :: xycor2
    real(stnd), dimension(q,3)      :: ystat
    real(stnd), dimension(q,4)      :: ystat2
    real(stnd), dimension(n,m,3)    :: xstat
    real(stnd), dimension(n,m,4)    :: xstat2
    real(stnd), dimension(n,m,p)    :: x, xt, xt2
    real(stnd), dimension(q,p)      :: y, yt, yt2
!
    integer(i4b) :: i, j, k, l, k2, inc
!
    logical(lgl)                   :: first, last
    logical(lgl), dimension(n,m,p) :: xmask
    logical(lgl), dimension(q,p)   :: ymask
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='comp_cor_miss'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TESTS.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = sqrt( epsilon(eps) )
!
!   GENERATE A RANDOM OBSERVATION ARRAY x WITH MISSING VALUES.
!
    call random_number( x )
!
    xmask = x>0.05_stnd
    where ( .not.xmask ) x = miss
!
!   COMPUTE THE MEANS, VARIANCES OF x  .
!
    xstat(:,:,3_i4b) = count( xmask, dim=3_i4b )
!
    xstat(:,:,1_i4b) = sum( x, dim=3_i4b, mask=xmask )/xstat(:,:,3_i4b)
    where ( xmask ) 
        xt = x - spread( xstat(:,:,1_i4b), dim=3_i4b, ncopies=p )
        xt2 = xt*xt
    elsewhere
        xt = miss
        xt2 = miss
    end where
    xstat(:,:,2_i4b) = sum( xt2, dim=3_i4b, mask=xmask )/xstat(:,:,3_i4b)
!
!   GENERATE A RANDOM OBSERVATION VECTOR y WITH MISSING VALUES .
!
    call random_number( y )
!
    ymask = y>0.05_stnd
    where ( .not.ymask ) y = miss
!
!   COMPUTE THE MEANS, VARIANCE OF y  .
!
    ystat(:,3_i4b) = count( ymask, dim=2_i4b )
!
    ystat(:,1_i4b) = sum( y, dim=2_i4b, mask=ymask )/ystat(:,3_i4b)
    where( ymask )
        yt = y - spread( ystat(:,1_i4b), dim=2_i4b, ncopies=p )
        yt2 = yt*yt
    elsewhere
        yt = miss
        yt2 = miss
    end where
    ystat(:,2_i4b) = sum( yt2, dim=2_i4b, mask=ymask )/ystat(:,3_i4b)
!
!   COMPUTE THE CORRELATIONS BETWEEN x AND y  .
!
    xycor = zero
    do k = 1_i4b, p
        do i = 1_i4b, q
            where ( xmask(:,:,k) .and. ymask(i,k) )
                xycor(:,:,i,1_i4b) = xycor(:,:,i,1_i4b) + yt(i,k)*xt(:,:,k)
                xycor(:,:,i,2_i4b) = xycor(:,:,i,2_i4b) + one
            end where
        end do
    end do
    xycor(:,:,:,1_i4b) = xycor(:,:,:,1_i4b)/xycor(:,:,:,2_i4b)
    do i = 1_i4b, q
        xycor(:,:,i,1_i4b) = xycor(:,:,i,1_i4b)/( sqrt( ystat(i,2_i4b)*xstat(:,:,2_i4b) )  )
    end do
!
    where ( xycor(:,:,:,1_i4b)<-one ) xycor(:,:,:,1_i4b) = -one
    where ( xycor(:,:,:,1_i4b)>one ) xycor(:,:,:,1_i4b) = one
!
    do i = 1_i4b, 8_i4b
!
        inc = merge( 1_i4b, p, i<=4_i4b )
!
        select case( i )
!
            case( 1_i4b, 5_i4b )
!
                xstat2(:,:,:3_i4b) = xstat(:,:,:3_i4b)
                ystat2(:,:3_i4b) = ystat(:,:3_i4b)
                xycor2(:,:,:,:2_i4b) = xycor(:,:,:,:2_i4b)
!
!               TESTS 1 AND 5 :
!
!               COMPUTE THE STATISTICS WITH comp_cor_miss_rt .
!
                k2 = 0_i4b
                do k = 1_i4b, p, inc
!
                    k2 = min( k2+inc, p )
!
                    first = k==1_i4b
                    last = k2==p
!
                    call comp_cor_miss( x(:,:,k:k2), y(1_i4b,k:k2), first, last,                     &
                                        xstat2, ystat2(1_i4b,:), xycor2(:,:,1_i4b,:), xymiss=miss )
                end do
!
            case( 2_i4b, 6_i4b )
!
!               TESTS 2 AND 6 :
!
!
!               COMPUTE THE STATISTICS OF WITH comp_cor_miss_rm .
!
                do j = 1_i4b, m
                    k2 = 0_i4b
                    do k = 1_i4b, p, inc
!
                        k2 = min( k2+inc, p )
!
                        first = k==1_i4b
                        last = k2==p
!
                        call comp_cor_miss( x(:,j,k:k2), y(1_i4b,k:k2), first, last,                            &
                                            xstat2(:,j,:), ystat2(1_i4b,:), xycor2(:,j,1_i4b,:), xymiss=miss )
                    end do
                end do
!
            case( 3_i4b, 7_i4b )
!
!               TESTS 3 AND 7 :
!
!
!               COMPUTE THE STATISTICS WITH comp_cor_miss_rv .
!
                do l = 1_i4b, n
                    do j = 1_i4b, m
                        k2 = 0_i4b
                        do k = 1_i4b, p, inc
!
                            k2 = min( k2+inc, p )
!
                            first = k==1_i4b
                            last = k2==p
!
                            call comp_cor_miss( x(l,j,k:k2), y(1_i4b,k:k2), first, last,                              &
                                                xstat2(l,j,:), ystat2(1_i4b,:), xycor2(l,j,1_i4b,:), xymiss=miss )
                        end do
                    end do
                end do
!
            case( 4_i4b, 8_i4b )
!
!               TESTS 4 AND 8 :
!
!
!               COMPUTE THE STATISTICS OF WITH comp_cor_miss_rm_rm .
!
                do j = 1_i4b, m
                    k2 = 0_i4b
                    do k = 1_i4b, p, inc
!
                        k2 = min( k2+inc, p )
!
                        first = k==1_i4b
                        last = k2==p
!
                        call comp_cor_miss( x(:,j,k:k2), y(:,k:k2), first, last,                        &
                                            xstat2(:,j,:), ystat2, xycor2(:,j,:,:), xymiss=miss )
                    end do
                end do
!
        end select
!
!       CHECK THAT THE TWO SETS OF STATISTICS AGREE.
!
        err_xstat = maxval( abs( (xstat-xstat2(:,:,:3))/xstat2(:,:,:3) ) )
        err_ystat = maxval( abs( (ystat-ystat2(:,:3))/ystat2(:,:3) ) )
        err_cor  = maxval( abs( xycor-xycor2(:,:,:,:2) ) )
!
!       PRINT THE RESULTS OF THE TESTS.
!
        err = max( err_xstat, err_ystat, err_cor )
!
        if ( err<=eps ) then
            write (prtunit,*) 'Test ',i,' is correct'
        else
            write (prtunit,*) 'Test ',i,' is incorrect'
            write (prtunit,*) '=> ',err,' > ',eps
        end if
!
    end do
!
!
! END OF PROGRAM test_comp_cor_miss
! =================================
!
end program test_comp_cor_miss
