program test_comp_cor_miss2
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine comp_cor_miss2 in module Mul_Stat_Procedures .
!                                                                              
! LATEST REVISION : 20/04/2010
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, machtiny, comp_cor_miss2
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=26, m=20, p=50
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                      :: err_xstat, err_ystat, err_cor, err, eps, miss=-999.99_stnd
    real(stnd), dimension(n,m)      :: xycor, xycor2, xyn, xyn2
    real(stnd), dimension(n,m,2)    :: xstat, xstat2, ystat, ystat2
    real(stnd), dimension(n,m,p)    :: x, xt, xt2, yt, yt2
    real(stnd), dimension(p)        :: y
!
    integer(i4b) :: i, j, k, l, k2, inc
!
    logical(lgl)                   :: first, last
    logical(lgl), dimension(n,m,p) :: xmask
    logical(lgl), dimension(p)     :: ymask
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='comp_cor_miss2'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TESTS.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = sqrt( epsilon(eps) )
!
!   GENERATE A RANDOM OBSERVATION ARRAY x WITH MISSING VALUES.
!
    call random_number( x )
!
    xmask = x>0.05_stnd
    where ( .not.xmask ) x = miss
!
!   GENERATE A RANDOM OBSERVATION VECTOR y WITH MISSING VALUES .
!
    call random_number( y )
!
    ymask = y>0.05_stnd
    where ( .not.ymask ) y = miss
    yt = spread(spread(y,1,n),2,m)
!
    xmask = xmask .and. spread(spread(ymask,1,n),2,m)
!
!   COMPUTE THE MEANS, VARIANCES OF x AND y  .
!
    xyn  = count( xmask, dim=3_i4b )
!
    xstat(:,:,1_i4b) = sum( x,  dim=3_i4b, mask=xmask )/xyn
    ystat(:,:,1_i4b) = sum( yt, dim=3_i4b, mask=xmask )/xyn
    where ( xmask ) 
        xt = x - spread( xstat(:,:,1_i4b), dim=3_i4b, ncopies=p )
        xt2 = xt*xt
        yt = yt - spread( ystat(:,:,1_i4b), dim=3_i4b, ncopies=p )
        yt2 = yt*yt
    elsewhere
        xt = miss
        xt2 = miss
        yt2 = miss
    end where
    xstat(:,:,2_i4b) = sum( xt2, dim=3_i4b, mask=xmask )
    ystat(:,:,2_i4b) = sum( yt2, dim=3_i4b, mask=xmask )
!
!   COMPUTE THE CORRELATIONS BETWEEN x AND y  .
!
    xycor = SUM(yt*xt, mask=xmask, dim=3_i4b)/( SQRT( ystat(:,:,2_i4b)*xstat(:,:,2_i4b) ) + machtiny )
!
    xstat(:,:,2_i4b) = xstat(:,:,2_i4b)/xyn
    ystat(:,:,2_i4b) = ystat(:,:,2_i4b)/xyn
!
    do i = 1_i4b, 6_i4b
!
        inc = merge( 1_i4b, p, i<=3_i4b )
!
        select case( i )
!
            case( 1_i4b, 4_i4b )
!
!               TESTS 1 AND 4 :
!
!               COMPUTE THE STATISTICS WITH comp_cor_miss2_rt .
!
                k2 = 0_i4b
                do k = 1_i4b, p, inc
!
                    k2 = min( k2+inc, p )
!
                    first = k==1_i4b
                    last = k2==p
!
                    call comp_cor_miss2( x(:,:,k:k2), y(k:k2), first, last,   &
                                        xstat2, ystat2, xycor2, xyn2, xymiss=miss )
                end do
!
            case( 2_i4b, 5_i4b )
!
!               TESTS 2 AND 5 :
!
!
!               COMPUTE THE STATISTICS OF x1 WITH comp_cor_miss2_rm .
!
                do j = 1_i4b, m
                    k2 = 0_i4b
                    do k = 1_i4b, p, inc
!
                        k2 = min( k2+inc, p )
!
                        first = k==1_i4b
                        last = k2==p
!
                        call comp_cor_miss2( x(:,j,k:k2), y(k:k2), first, last, xstat2(:,j,:),      &
                                             ystat2(:,j,:), xycor2(:,j), xyn2(:,j), xymiss=miss )
                    end do
                end do
!
            case( 3_i4b, 6_i4b )
!
!               TESTS 3 AND 6 :
!
!
!               COMPUTE THE STATISTICS WITH comp_cor_miss2_rv .
!
                do l = 1_i4b, n
                    do j = 1_i4b, m
                        k2 = 0_i4b
                        do k = 1_i4b, p, inc
!
                            k2 = min( k2+inc, p )
!
                            first = k==1_i4b
                            last = k2==p
!
                            call comp_cor_miss2( x(l,j,k:k2), y(k:k2), first, last, xstat2(l,j,:),      &
                                                 ystat2(l,j,:), xycor2(l,j), xyn2(l,j), xymiss=miss )
                        end do
                    end do
                end do
!
        end select
!
!       CHECK THAT THE TWO SETS OF STATISTICS AGREE.
!
        err_xstat = maxval( abs( (xstat-xstat2)/xstat2 ) )
        err_ystat = maxval( abs( (ystat-ystat2)/ystat2 ) )
        err_cor  = maxval( abs( xycor-xycor2 ) )
!
!       PRINT THE RESULTS OF THE TESTS.
!
        err = max( err_xstat, err_ystat, err_cor )
!
        if ( err<=eps ) then
            write (prtunit,*) 'Test ',i,' is correct'
        else
            write (prtunit,*) 'Test ',i,' is incorrect'
            write (prtunit,*) '=> ',err,' > ',eps
        end if
    end do
!
!
! END OF PROGRAM test_comp_cor_miss2
! ==================================
!
end program test_comp_cor_miss2
