program test_comp_cormat
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine comp_cormat in module Mul_Stat_Procedures .
!                                                                              
! LATEST REVISION : 20/04/2010
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, true, false, one, comp_cormat, triangle
#ifdef _MATMUL
    use Statpack, only : matmul=>matmul2
#endif
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, m=20, p=50
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                         :: err_mean, err_std, err_cor, err, eps, xn
    real(stnd), dimension(m,m)         :: cor1, cov1, cor2
    real(stnd), dimension(m)           :: mean1, mean2, std1, std2
    real(stnd), dimension((m*(m+1))/2) :: corp1, covp1, corp2
    real(stnd), dimension(m,p)         :: x, xt
!
    integer(i4b) :: i, k, k2, inc
!
    logical(lgl) :: first, last, cov
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='comp_cormat'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = sqrt( epsilon(eps) )
!
!   GENERATE A RANDOM OBSERVATION ARRAY x .
!
    call random_number( x )
!
!   COMPUTE THE MEANS AND STANDARD-DEVIATIONS OF x  .
!
    mean1 = sum( x, dim=2_i4b )/p
    xt = x - spread( mean1, dim=2_i4b, ncopies=p )
    std1 = sqrt( sum( xt**2, dim=2_i4b )/p )
!
!   COMPUTE THE COVARIANCE  MATRIX  .
!
    cov1 = matmul( xt, transpose( xt ) )/p
    covp1 = pack( cov1, mask=triangle( true, m, m, extra=1_i4b) )
!
!   COMPUTE THE CORRELATION MATRIX .
!
    xt = xt*spread( one/std1, dim=2_i4b, ncopies=p )
    cor1 = matmul( xt, transpose( xt ) )/p
    corp1 = pack( cor1, mask=triangle( true, m, m, extra=1_i4b) )
!
    cov = false
!
    do i = 1_i4b, 8_i4b
!
        cov = .not.cov
        inc = merge( 1_i4b, p, i<=4_i4b )
!
        select case( i )
!
            case( 1_i4b, 2_i4b, 5_i4b, 6_i4b  )
!
!               TESTS 1, 2, 5 AND 6 :
!
!               COMPUTE THE COVARIANCE OR CORRELATION MATRIX WITH comp_cormat .
!
                k2 = 0_i4b
                do k = 1_i4b, p, inc
!
                    k2 = min( k2+inc, p )
!
                    first = k==1_i4b
                    last = k2==p
!
                    call comp_cormat( x(:,k:k2), first, last, mean2, cor2, xn,    &
                                      xstd=std2, cov=cov, fill=true )
                end do
!
                if ( cov ) then
                    err_cor  = maxval( abs( cov1-cor2 ) )
                else
                    err_cor  = maxval( abs( cor1-cor2 ) )
                end if
!
            case( 3_i4b, 4_i4b, 7_i4b, 8_i4b )
!
!               TESTS 3, 4, 7 AND 8 :
!
!               COMPUTE THE COVARIANCE OR CORRELATION MATRIX WITH comp_cormat_b .
!
                k2 = 0_i4b
                do k = 1_i4b, p, inc
!
                    k2 = min( k2+inc, p )
!
                    first = k==1_i4b
                    last = k2==p
!
                    call comp_cormat( x(:,k:k2), first, last, mean2, corp2, xn,    &
                                      xstd=std2, cov=cov )
                end do
!
                if ( cov ) then
                    err_cor  = maxval( abs( covp1-corp2 ) )
                else
                    err_cor  = maxval( abs( corp1-corp2 ) )
                end if
!
        end select
!
!       CHECK THAT THE TWO SETS OF STATISTICS AGREE.
!
        err_mean = maxval( abs( (mean1-mean2)/mean2 ) )
        err_std  = maxval( abs( (std1-std2)/std2 ) )
!
!       PRINT THE RESULTS OF THE TESTS.
!
        err = max( err_mean, err_std, err_cor )
!
        if ( err<=eps ) then
            write (prtunit,*) 'Test ',i,' is correct'
        else
            write (prtunit,*) 'Test ',i,' is incorrect'
            write (prtunit,*) '=> ',err,' > ',eps
        end if
!
    end do
!
!
! END OF PROGRAM test_comp_cormat
! ===============================
!
end program test_comp_cormat
