program test_comp_mca_miss2
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine comp_mca_miss2 in module Mul_Stat_Procedures .
!                                                                              
! LATEST REVISION : 28/08/2020
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, false, zero, one, comp_mca_miss2
#ifdef _MATMUL
    use Statpack, only : matmul=>matmul2
#endif
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, mx=40, my=20, m=min(mx,my), p=50, nsvd=3, mxy=mx+my
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                         :: err_xstat, err_ystat, err_svd, err, eps, miss=-999.99_stnd
    real(stnd), dimension(mx,my)       :: xycor, xycov
    real(stnd), dimension(mx,my,4)     :: xycor2
    real(stnd), dimension(mx,nsvd)     :: res
    real(stnd), dimension(mxy,nsvd)    :: xysingvec
    real(stnd), dimension(mx,3)        :: xstat1
    real(stnd), dimension(mx,4)        :: xstat2
    real(stnd), dimension(my,3)        :: ystat1
    real(stnd), dimension(my,4)        :: ystat2
    real(stnd), dimension(m)           :: xysingval
    real(stnd), dimension(mx,p)        :: x, xt
    real(stnd), dimension(my,p)        :: y, yt
!
    integer(i4b) :: i, k, k2, inc, maxiter=3
!
    logical(lgl)                  :: first, last, cov, failure
    logical(lgl), dimension(mx,p) :: xmask
    logical(lgl), dimension(my,p) :: ymask
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='comp_mca_miss2'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = sqrt( epsilon(eps) )
!
!   GENERATE A RANDOM OBSERVATION ARRAY x WITH MISSING VALUES.
!
    call random_number( x )
!
    xmask = x>0.05_stnd
    where ( .not.xmask ) x = miss
!
!   COMPUTE THE MEANS AND STANDARD-DEVIATIONS OF x  .
!
    xstat1(:,3_i4b) = count( xmask, dim=2_i4b )
!
    xstat1(:,1_i4b) = sum( x, dim=2_i4b, mask=xmask )/xstat1(:,3_i4b)
    where ( xmask ) 
        xt = x - spread( xstat1(:,1_i4b), dim=2_i4b, ncopies=p )
    elsewhere
        xt = zero
    end where
    xstat1(:,2_i4b) = sqrt( sum( xt**2, dim=2_i4b )/xstat1(:,3_i4b) )
!
!   GENERATE A RANDOM OBSERVATION ARRAY y WITH MISSING VALUES.
!
    call random_number( y )
!
    ymask = y>0.04_stnd
    where ( .not.ymask ) y = miss
!
!   COMPUTE THE MEANS AND STANDARD-DEVIATIONS OF y  .
!
    ystat1(:,3_i4b) = count( ymask, dim=2_i4b )
!
    ystat1(:,1_i4b) = sum( y, dim=2_i4b, mask=ymask )/ystat1(:,3_i4b)
    where ( ymask ) 
        yt = y - spread( ystat1(:,1_i4b), dim=2_i4b, ncopies=p )
    elsewhere
        yt = zero
    end where
    ystat1(:,2_i4b) = sqrt( sum( yt**2, dim=2_i4b )/ystat1(:,3_i4b) )
!
!   COMPUTE THE COVARIANCE MATRIX BETWEEN x AND y .
!
    xycov = matmul( xt, transpose( yt ) )
!
    where ( xmask ) xt = one
!
    where ( ymask ) yt = one
!
    xycor2(:,:,1_i4b) = matmul( xt, transpose( yt ) )
    xycov(:,:) = xycov(:,:) / xycor2(:,:,1_i4b)
!
!   COMPUTE THE CORRELATION MATRIX BETWEEN x AND y .
!
    xycor = xycov*spread( one/xstat1(:,2_i4b), dim=2_i4b, ncopies=my )
    xycor = xycor*spread( one/ystat1(:,2_i4b), dim=1_i4b, ncopies=mx )
!
    cov = false
!
    do i = 1_i4b, 4_i4b
!
        cov = .not.cov
        inc = merge( 1_i4b, p, i<=2_i4b )
!
!       COMPUTE SINGULAR VECTORS OF THE COVARIANCE OR CORRELATION MATRIX .
!
        k2 = 0_i4b
!
        do k = 1_i4b, p, inc
!
            k2 = min( k2+inc, p )
!
            first = k==1_i4b
            last = k2==p
!
            call comp_mca_miss2( x(:,k:k2), y(:,k:k2), first, last, xstat2, ystat2, xycor2, miss, failure,    &
                                 cov=cov, maxiter=maxiter, xysingval=xysingval, xysingvec=xysingvec )
        end do
!
        if ( cov ) then
!
!
!           CHECK FOR SMALL RESIDUALS OF THE EXPRESSION:
!                xycov*ysingvec(:,:nsvd) - xsingvec(:,:nsvd)*diag(xysingval(:nsvd)).
!
            res(:mx,:nsvd) = matmul(xycov,xysingvec(mx+1:mxy,:nsvd)) - xysingvec(:mx,:nsvd)*spread(xysingval(:nsvd),1,mx)
!
        else
!
!           CHECK FOR SMALL RESIDUALS OF THE EXPRESSION:
!                xycor*ysingvec(:,:nsvd) - xsingvec(:,:nsvd)*diag(xysingval(:nsvd)).
!
            res(:mx,:nsvd) = matmul(xycor,xysingvec(mx+1:mxy,:nsvd)) - xysingvec(:mx,:nsvd)*spread(xysingval(:nsvd),1,mx)
!
        end if
!
        err_svd = sum( abs( res(:mx,:nsvd) ) )/sum( xysingval(:nsvd) )
!
!       CHECK THAT THE TWO SETS OF UNIVARIATE STATISTICS AGREE.
!
        err_xstat  = maxval( abs( (xstat1(:,:2_i4b)-xstat2(:,:2_i4b))/xstat2(:,:2_i4b) ) )
        err_ystat  = maxval( abs( (ystat1(:,:2_i4b)-ystat2(:,:2_i4b))/ystat2(:,:2_i4b) ) )
!
!       PRINT THE RESULTS OF THE TESTS.
!
        err = max(err_xstat, err_ystat, err_svd )
!
        if ( err<=eps .and. .not.failure ) then
            write (prtunit,*) 'Test ',i,' is correct'
        else
            write (prtunit,*) 'Test ',i,' is incorrect'
            write (prtunit,*) '=> ',err,' > ',eps
        end if
!
    end do
!
!
! END OF PROGRAM test_comp_mca_miss2
! ==================================
!
end program test_comp_mca_miss2
