program test_comp_pc
!
!
! Purpose
! =======
!
!   This program is intended to test subroutines  comp_eof, comp_eof2 and comp_pc
!   in module Mul_Stat_Procedures .
!                                                                              
! LATEST REVISION : 23/05/2012
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, false, one, comp_eof, comp_eof2, comp_pc
#ifdef _MATMUL
    use Statpack, only : matmul=>matmul2
#endif
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, m=20, p=50, mp=(m*(m+1))/2
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                         :: err_mean, err_std, err_pc, err, eps, xn
    real(stnd), dimension(m)           :: mean1, mean2, std1, std2, eigval
    real(stnd), dimension(mp)          :: corp
    real(stnd), dimension(m,m)         :: eigvec
    real(stnd), dimension(m,p)         :: x, xt, xt_std
    real(stnd), dimension(p,m)         :: pc
!
    integer(i4b) :: i, k, k2, inc, maxiter=3
!
    logical(lgl) :: first, last, cov, failure
!
    character    :: sort = 'd'
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='comp_pc'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = sqrt( epsilon(eps) )
!
!   GENERATE A RANDOM OBSERVATION ARRAY x .
!
    call random_number( x )
!
!   COMPUTE THE MEANS AND STANDARD-DEVIATIONS OF x  .
!
    mean1 = sum( x, dim=2_i4b )/p
    xt = x - spread( mean1, dim=2_i4b, ncopies=p )
    std1 = sqrt( sum( xt**2, dim=2_i4b )/p )
!
!   COMPUTE STANDARDIZED VARIABLES .
!
    std2 = one/std1
    xt_std = xt*spread( std2, dim=2_i4b, ncopies=p ) 
!
    cov = false
!
!   TEST OF comp_pc_rv .
!
    do i = 1_i4b, 4_i4b
!
        cov = .not.cov
        inc = merge( 1_i4b, p, i<=2_i4b )
!
!       COMPUTE EOFs FROM THE COVARIANCE OR CORRELATION MATRIX .
!
        k2 = 0_i4b
        do k = 1_i4b, p, inc
!
            k2 = min( k2+inc, p )
!
            first = k==1_i4b
            last = k2==p
!
            call comp_eof2( x(:,k:k2), first, last, eigval, corp,  xn, failure,   &
                            cov=cov, maxiter=maxiter, xmean=mean2, xstd=std2, xeigvec=eigvec(:,:1_i4b) )
        end do
!
        if ( cov ) then
!
            k2 = 0_i4b
            do k = 1_i4b, p, inc
!
                k2 = min( k2+inc, p )
!
                call comp_pc( x(:,k:k2), eigvec(:,1_i4b), pc(k:k2,1_i4b),     &
                             xmean=mean2                                 )
            end do
!
!           CHECK FOR SMALL RESIDUALS OF THE EXPRESSION xt**t*eigvec - pc
!           WHERE eigvec THE EIGENVECTORS OF THE COVARIANCE MATRIX, xt THE CENTERED DATA
!           AND pc THE UNNORMALIZED PRINCIPAL COMPONENTS.
!
            err_pc =         &
            sum( abs( matmul(transpose(xt),eigvec(:,1_i4b))-pc(:,1_i4b)) )/eigval(1_i4b)
        else
!
            k2 = 0_i4b
            do k = 1_i4b, p, inc
!
                k2 = min( k2+inc, p )
!
                call comp_pc( x(:,k:k2), eigvec(:,1_i4b), pc(k:k2,1_i4b),      &
                              xmean=mean2, xstd=std2           )
            end do
!
!           CHECK FOR SMALL RESIDUALS OF THE EXPRESSION xt_std**t*eigvec - pc
!           WHERE eigvec THE EIGENVECTORS OF THE CORRELATION MATRIX, xt_std THE STANDARDIZED DATA
!           AND pc THE UNNORMALIZED PRINCIPAL COMPONENTS.
!
            err_pc =         &
            sum( abs(matmul(transpose(xt_std),eigvec(:,1_i4b))-pc(:,1_i4b)) )/eigval(1_i4b)
!
        end if
!
!       CHECK THAT THE TWO SETS OF UNIVARIATE STATISTICS AGREE.
!
        err_mean = maxval( abs( (mean1-mean2)/mean2 ) )
        err_std  = maxval( abs( (std1-std2)/std2 ) )
!
!       PRINT THE RESULTS OF THE TESTS.
!
        if ( max(err_mean, err_std, err_pc )<=eps .and. .not.failure ) then
            write (prtunit,*) 'Test ',i,' is correct'
        else
            write (prtunit,*) 'Test ',i,' is incorrect'
        end if
    end do
!
!   TEST OF comp_pc_rm .
!
    do i = 5_i4b, 8_i4b
!
        cov = .not.cov
        inc = merge( 1_i4b, p, i<=6_i4b )
!
!       COMPUTE EOFs FROM THE COVARIANCE OR CORRELATION MATRIX .
!
        k2 = 0_i4b
        do k = 1_i4b, p, inc
!
            k2 = min( k2+inc, p )
!
            first = k==1_i4b
            last = k2==p
!
            call comp_eof( x(:,k:k2), first, last, eigval, eigvec, xn, failure,   &
                           cov=cov, sort=sort, xmean=mean2, xstd=std2      )
        end do
!
        if ( cov ) then
!
            k2 = 0_i4b
            do k = 1_i4b, p, inc
!
                k2 = min( k2+inc, p )
!
                first = k==1_i4b
                last = k2==p
!
                call comp_pc( x(:,k:k2), eigvec, pc(k:k2,:),          &
                             xmean=mean2                              )
            end do
!
!           CHECK FOR SMALL RESIDUALS OF THE EXPRESSION xt**t*eigvec - pc
!           WHERE eigvec THE EIGENVECTORS OF THE COVARIANCE MATRIX, xt THE CENTERED DATA
!           AND pc THE UNNORMALIZED PRINCIPAL COMPONENTS.
!
            err_pc =         &
            sum( abs(matmul(transpose(xt),eigvec)-pc) )/sum( eigval )
        else
!
            k2 = 0_i4b
            do k = 1_i4b, p, inc
!
                k2 = min( k2+inc, p )
!
                first = k==1_i4b
                last = k2==p
!
                call comp_pc( x(:,k:k2), eigvec, pc(k:k2,:),   &
                              xmean=mean2, xstd=std2           )
            end do
!
!           CHECK FOR SMALL RESIDUALS OF THE EXPRESSION xt_std**t*eigvec - pc
!           WHERE eigvec THE EIGENVECTORS OF THE CORRELATION MATRIX, xt_std THE STANDARDIZED DATA
!           AND pc THE UNNORMALIZED PRINCIPAL COMPONENTS.
!
            err_pc =         &
            sum( abs(matmul(transpose(xt_std),eigvec)-pc) )/sum( eigval )
!
        end if
!
!       CHECK THAT THE TWO SETS OF UNIVARIATE STATISTICS AGREE.
!
        err_mean = maxval( abs( (mean1-mean2)/mean2 ) )
        err_std  = maxval( abs( (std1-std2)/std2 ) )
!
!       PRINT THE RESULTS OF THE TESTS.
!
        err = max( err_mean, err_std, err_pc )
!
        if ( err<=eps .and. .not.failure ) then
            write (prtunit,*) 'Test ',i,' is correct'
        else
            write (prtunit,*) 'Test ',i,' is incorrect'
            write (prtunit,*) '=> ',err,' > ',eps
        end if
!
    end do
!
!
! END OF PROGRAM test_comp_pc
! ===========================
!
end program test_comp_pc
