program test_fft_row
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine FFT_ROW in module FFT_Procedures.
!                                                                              
!                                                                              
! LATEST REVISION : 21/05/2018
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, n1_def, n2_def, n3_def,  &
                         true, false, c1000, half, init_fft, &
                         fft_row, end_fft, lamch, norm
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n1=n1_def, n2=n2_def, n3=n3_def
!   
    real(stnd), parameter  :: fudge=c1000
!
    character(len=*), parameter :: name_proc='fft_row'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                              :: err, eps, ulp, safmin, tmp
    real(stnd), allocatable, dimension(:,:) :: x, y
!
    complex(stnd), allocatable, dimension(:,:) :: a, c
!   
    integer(i4b), dimension(2) :: init_vec
    integer(i4b)               :: j, m, n, success
    integer                    :: iok
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TESTS.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    ulp    = epsilon( err )
    eps    = fudge*ulp
    safmin = lamch( 'S' )
!
    success = 0_i4b
    m       = 0_i4b
!
    do n = n1, n2, n3
!
!       ALLOCATE WORK ARRAYS.
!
        allocate( a(n,n), c(n,n), x(n,n), y(n,n), stat=iok )
!
        if ( iok/=0 ) cycle
!
!       GENERATE A COMPLEX RANDOM NUMBER ARRAY.
!
        call random_number( x(:n,:n) )
!
        x(:n,:n) = x(:n,:n) - half
!
        call random_number( tmp )
!
        if ( tmp>safmin ) then
            x(:n,:n) = x(:n,:n)/tmp
        end if
!
        call random_number( y(:n,:n) )
!
        y(:n,:n) = y(:n,:n) - half
!
        call random_number( tmp )
!
        if ( tmp>safmin ) then
            y(:n,:n) = y(:n,:n)/tmp
        end if
!
        a(:n,:n) = cmplx( x(:n,:n), y(:n,:n), kind=stnd )
!
        deallocate( x, y )
!
!       SAVE THE COMPLEX RANDOM NUMBER ARRAY.
!
        c(:n,:n) = a(:n,:n)
!
        init_vec(:2_i4b) = (/ n, n /)
!
        do j = 1_i4b, 2_i4b
!
            m = m + 1_i4b
!
            select case( j )
!
            case( 1_i4b)
!
!               TEST 1 : TEST 1D FFT_ROW SUBROUTINE.
!
!               INITIALIZE THE FFT SUBROUTINE.
!
                call init_fft( n )
!
!               TRANSFORM AND THEN INVERT THE SEQUENCE BACK.
!
                call fft_row( a(:n,1_i4b), forward=true  )
                call fft_row( a(:n,1_i4b), forward=false )
!
!               CHECK THAT INVERSE(TRANSFORM(SEQUENCE)) = SEQUENCE.
!
                a(:n,1_i4b) = a(:n,1_i4b) - c(:n,1_i4b)
!
                err = norm( a(:n,1_i4b) )/norm( c(:n,1_i4b) )
!
                a(:n,1_i4b) = c(:n,1_i4b)
!
            case( 2_i4b)
!
!               TEST 2 : TEST 2D FFT_ROW SUBROUTINE.
!
!               INITIALIZE THE FFT SUBROUTINE.
!
                call init_fft( init_vec(:2), dim=2_i4b )
!
!               TRANSFORM AND THEN INVERT THE SEQUENCE BACK.
!
                call fft_row( a(:n,:n), forward=true  )
                call fft_row( a(:n,:n), forward=false )
!
!               CHECK THAT INVERSE(TRANSFORM(SEQUENCE)) = SEQUENCE.
!
                a(:n,:n) = a(:n,:n) - c(:n,:n)
!
                err = norm( a(:n,:n) )/norm( c(:n,:n) )
!
                a(:n,:n) = c(:n,:n)
!
            end select
!
!           PRINT THE RESULTS OF THE TEST.
!
            if ( err<=eps ) then
!
                success = success + 1_i4b
!
            else
!
                write (prtunit,*) 'Test ',m,' is incorrect'
!
                write (prtunit,*) '=> ',err,' > ',eps,' , n= ', n,' , case= ', j
!
            end if
!
        end do
!
!       DEALLOCATE WORK ARRAYS.
!
        call end_fft()
!
        deallocate( a, c )
!
    end do
!
    if ( m/=success ) then
        write (prtunit,*) m - success, 'tests are incorrect'
    end if
!
    write (prtunit,*) success, 'tests are correct'
!
!
! END OF PROGRAM test_fft_row
! ===========================
!
end program test_fft_row
