program test_hwfilter2
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine HWFILTER2
!   in module Time_Series_Procedures .
!                                                                              
!                                                                              
! LATEST REVISION : 02/03/2010
!
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, hwfilter2, one
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n1=160, n2=123
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                              :: err, win, eps
    real(stnd), allocatable, dimension(:,:) :: x, x2, x3
!   
    integer(i4b) :: pl, ph, j, i, mi
    integer      :: iok
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='hwfilter2'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TESTS.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = sqrt( epsilon(err) )
!
!   GENERATE A REAL RANDOM NUMBER ARRAY.
!
    allocate( x(n1,n2), x2(n1,n2), x3(n1,n2), stat=iok )
    if ( iok/=0 ) then
        write (prtunit,*)    &
        'Problem in attempt to allocate memory for tests 1 to 6'
    end if
!
    call random_number( x(:n1,:n2) )
!
!   SAVE THE REAL RANDOM NUMBER ARRAY.
!
    x2(:n1,:n2) = x(:n1,:n2)
    x3(:n1,:n2) = x(:n1,:n2)
!
    pl  = 10_i4b
    ph  = 20_i4b
    win = one
!
    do j = 1_i4b, 6_i4b
!
        if ( j<=3 ) then
            i = j
        else
            i = j - 3_i4b
        end if
!
        mi = -i
!
        select case( j )
!
            case( 1_i4b:3_i4b )
!
!               TESTS 1, 2 AND 3 : TEST 1D HWFILTER2 .
!
                call hwfilter2( x2(:n1,1), pl, ph, win=win, trend=mi )
                call hwfilter2( x3(:n1,1), ph, pl, win=win, trend=i  )
!
                x2(:n1,1) = x2(:n1,1) + x3(:n1,1)
!
                err = maxval(abs(x(:n1,1)-x2(:n1,1)))/maxval(abs(x(:n1,1)))
!
                x2(:n1,1) = x(:n1,1)
                x3(:n1,1) = x(:n1,1)
!
            case( 4_i4b:6_i4b )
!
!               TESTS 4, 5 AND 6 : TEST 2D HWFILTER2 .
!
                call hwfilter2( x2(:n1,:n2), pl, ph, win=win, trend=i  )
                call hwfilter2( x3(:n1,:n2), ph, pl, win=win, trend=mi  )
!
                x2(:n1,:n2) = x2(:n1,:n2) + x3(:n1,:n2)
!
                err = maxval(abs(x(:n1,:n2)-x2(:n1,:n2)))/maxval(abs(x(:n1,:n2)))
!
                x2(:n1,:n2) = x(:n1,:n2)
                x3(:n1,:n2) = x(:n1,:n2)
!
        end select
!
!       PRINT THE RESULTS OF THE TEST.
!
        if ( err<=eps ) then
            write (prtunit,*) 'Test ',j,' is correct'
        else
             write (prtunit,*) 'Test ',j,' is incorrect'
             write (prtunit,*) '=> ',err,' > ',eps
        end if
!
    end do
!
    deallocate( x, x2, x3 )
!
!
! END OF PROGRAM test_hwfilter2
! =============================
!
end program test_hwfilter2
