program test_pk_coef
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine PK_COEF in module Time_Series_Procedures.
!                                                                              
!                                                                              
! Further Details
! ===============
!
!    The program assumes that subroutine FREQ_FUNC in module Time_Series_Procedures is correct.
!                                                                              
! LATEST REVISION : 09/09/2015
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, half, one, c1_3, pk_coef, freq_func
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT .
!
    integer(i4b), parameter :: prtunit=6, n=2000
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                            :: diff, fch, fcl, fc, tmp
    real(stnd), dimension(n)              :: freqr, freq
    real(stnd), dimension(:), allocatable :: coef
!
    integer(i4b) :: k, pc, nmask, nmask2
!   
    integer :: iok
!   
    logical(lgl), dimension(n) :: mask, mask2
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='pk_coef'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   pc IS THE PERIOD OF OSCILLATION WITH A PEAK RESPONSE NEAR ONE.
!   pc IS EXPRESSED IN NUMBER OF OBSERVATIONS, I.E. pc==32(96) SELECT
!   A PERIOD OF 8 YRS FOR QUARTERLY (MONTHLY) DATA.
!
    pc  = 192_i4b
!
!   COMPUTE THE CORRESPONDING CUTOFF FREQUENCY.
!
    fc = one/real( pc, stnd )
!
!   NOW SELECT THE OPTIMAL NUMBER OF FILTER COEFFICIENTS k FOR THE LANCZOS FILTER.
!
    tmp = 2.3_stnd*max( real( pc, stnd ), one/(half-fc) )
    k = ceiling( tmp, i4b )
!
!   CHECK IF k IS ODD.
!
    if ( (k/2)*2==k  ) k = k + 1_i4b
!
!   ALLOCATE WORK ARRAY FOR THE FILTER COEFFICIENTS.
!
    allocate( coef(k), stat=iok )
!
    if ( iok==0 ) then
!
!       FUNCTION pk_coef COMPUTES THE K-TERM LEAST SQUARES APPROXIMATION TO AN
!       -IDEAL- BAND PASS FILTER WITH A PEAK RESPONSE NEAR ONE AT THE PERIOD pc (EG FREQUENCY FREQ=1/pc).
!
        coef(:k) = pk_coef( fc, k )
!
!       NOW COMPUTE THE TRANSFERT FUNCTION freqr(:) OF THE FILTER.
!
        call freq_func( n, coef(:k), freqr(:n), freq=freq(:n) )
!
!       COMPUTE THE TWO CUTOFF FREQUENCIES fcl AND fch .
!
        tmp  = c1_3/real( k + 1_i4b, stnd )
        fch  = fc - tmp
        fcl  = fc + tmp
!
        tmp = one/real( k, stnd )
!
        mask(:n) = freq(:n)>=( fch + tmp ) .and. freq(:n)<=( fcl - tmp )
        nmask    = count( mask(:n) )
!
        mask2(:n) = freq(:n)<( fch - tmp  ) .or. freq(:n)>( fcl + tmp )
        nmask2    = count( mask2(:n) )
!
        if ( nmask/=0_i4b .and. nmask2/=0_i4b ) then
!
!           CHECK THE PASS-BAND AND THE STOP-BAND OF THE FILTER.
!
            diff = sum( freqr(:n), mask=mask(:n))/real( nmask, stnd )           &
                   - sum( freqr(:n), mask=mask2(:n) )/real( nmask2, stnd )  
!
        else
!
            diff = -one
!
        end if
!
!       TEST THE ACCURACY OF THE RESULTS.
!
        if ( diff>0.95_stnd  ) then
            write (prtunit,*) 'Test ',1_i4b,' is correct'
        else
            write (prtunit,*) 'Test ',1_i4b,' is incorrect'
        end if
!
    else
!
        write (prtunit,*) 'Problem in attempt to allocate memory for test 1'
!
    end if
!
!
! END OF PROGRAM test_pk_coef
! ===========================
!
end program test_pk_coef
