program test_power_spectrum
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine POWER_SPECTRUM
!   in module Time_Series_Procedures .
!                                                                              
!                                                                              
! LATEST REVISION : 05/07/2010
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, true, false, power_spectrum, comp_mvs
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n1=10, n2=124
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                              :: err_mean, err_var, err, eps
    real(stnd), allocatable, dimension(:,:) :: x, psx
    real(stnd), allocatable, dimension(:)   :: xmean, xvar, xstd, xmean2, xvar2
!   
    integer(i4b) :: j, psn, trend, win
    integer      :: iok
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='power_spectrum'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TESTS.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = sqrt( epsilon(err) )
!
!   GENERATE A REAL RANDOM NUMBER ARRAY.
!
    psn = n2/2_i4b + 1_i4b
!
    allocate( x(n1,n2), psx(n1,psn), xmean(n1), xvar(n1), xstd(n1),    &
              xmean2(n1), xvar2(n1), stat=iok )
    if ( iok/=0 ) then
        write (prtunit,*)    &
        'Problem in attempt to allocate memory for tests 1 to 2'
    end if
!
    call random_number( x(:n1,:n2) )
!
    trend = 0_i4b
    win   = 2_i4b
!
    do j = 1_i4b, 2_i4b
!
        select case( j )
!
            case( 1_i4b )
!
!               TEST 1 : TEST 1D POWER SPECTRUM SUBROUTINE.
!
                psn = n1/2_i4b + 1_i4b
                call power_spectrum( x(:n1,1), psx(:psn,1), win=win, trend=trend, normpsd=false )
!
!               ESTIMATE THE MEAN AND VARIANCE OF THE SIGNAL THROUGH THE POWER SPECTRUM.
!
                xmean2(1) = sqrt( psx(1,1) )
                xvar2(1)  = sum( psx(2:psn,1) )
!
!               COMPUTE THE MEAN AND THE VARIANCE WITH SUBROUTINE comp_mvs .
!
                call comp_mvs( x(:n1,1), true, true, xmean(1), xvar(1), xstd(1) )
!
!               COMPARE THE TWO SETS OF STATISTICS.
!
                err_mean = abs( (xmean(1)-xmean2(1))/xmean(1) )
                err_var  = abs( (xvar(1)-xvar2(1))/xvar(1) )
!
            case( 2_i4b )
!
!               TEST 2 : TEST 2D POWER SPECTRUM SUBROUTINE.
!
                psn = n2/2_i4b + 1_i4b
                call power_spectrum( x(:n1,:n2), psx(:n1,:psn), win=win, trend=trend, normpsd=false )
!
!               ESTIMATE THE MEANS AND VARIANCES OF THE MULTICHANNEL SIGNAL THROUGH THE POWER SPECTRUM.
!
                xmean2(:n1) = sqrt( psx(:n1,1) )
                xvar2(:n1)  = sum( psx(:n1,2:psn), dim=2 )
!
!               COMPUTE THE MEANS AND THE VARIANCES WITH SUBROUTINE comp_mvs .
!
                call comp_mvs( x(:n1,:n2), true, true, xmean(:n1), xvar(:n1), xstd(:n1) )
!
!               COMPARE THE TWO SETS OF STATISTICS.
!
                err_mean = maxval( abs( (xmean(:n1)-xmean2(:n1))/xmean(:n1) ) )
                err_var  = maxval( abs( (xvar(:n1)-xvar2(:n1))/xvar(:n1) ) )
!
        end select
!
!       PRINT THE RESULTS OF THE TESTS.
!
        err = max( err_mean, err_var )
!
        if ( err<=eps ) then
            write (prtunit,*) 'Test ',j,' is correct'
        else
            write (prtunit,*) 'Test ',j,' is incorrect'
            write (prtunit,*) '=> ',err,' > ',eps
        end if
!
    end do
!
    deallocate( x, xmean, xvar, xstd, xmean2, xvar2 )
!
!
! END OF PROGRAM test_power_spectrum
! ==================================
!
end program test_power_spectrum
