program test_real_fft_forward
!
!
! Purpose
! =======
!
!   This program is intended to test subroutines REAL_FFT_FORWARD and REAL_FFT_BACKWARD
!   in module FFT_Procedures.
!                                                                              
!                                                                              
! LATEST REVISION : 26/09/2018
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, n1_def, n2_def, n3_def,                      &
                         c1000, half, four, real_fft_forward, real_fft_backward, &
                         lamch, norm
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n1=n1_def, n2=n2_def, n3=n3_def
!   
    real(stnd), parameter  :: fudge=c1000*four
!
    character(len=*), parameter :: name_proc='real_fft_forward'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                              :: err, eps, ulp, safmin, tmp
    real(stnd), allocatable, dimension(:,:) :: x, x2, xr, xi
!   
    integer(i4b) :: j, m, n, p, dim, dim2, success
    integer      :: iok
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TESTS.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    ulp    = epsilon( err )
    eps    = fudge*ulp
    safmin = lamch( 'S' )
!
    success = 0_i4b
    m       = 0_i4b
!
    do n = n1, n2, n3
!
        p = (n/2_i4b) + 1_i4b
!
!       ALLOCATE WORK ARRAYS.
!
        allocate( x(n,n), x2(n,n), xr(n,p), xi(n,p), stat=iok )
!
        if ( iok/=0 ) cycle
!
!       GENERATE A REAL RANDOM NUMBER ARRAY.
!
        call random_number( x2(:n,:n) )
!
        x2(:n,:n) = x2(:n,:n) - half
!
        call random_number( tmp )
!
        if ( tmp>safmin ) then
            x2(:n,:n) = x2(:n,:n)/tmp
        end if
!
        do j = 1_i4b, 3_i4b
!
            m = m + 1_i4b
!
            select case( j )
!
                case( 1_i4b )
!
!                   TEST 1: TEST 1D REAL_FFT_FORWARD AND REAL_FFT_BACKWARD .
!
                    x(:n,1_i4b) = x2(:n,1_i4b)
!
!                   TRANSFORM AND THEN INVERT THE SEQUENCE BACK.
!
                    call real_fft_forward( x(:n,1_i4b), xr(:p,1_i4b), xi(:p,1_i4b)  )
!
                    call real_fft_backward( xr(:p,1_i4b), xi(:p,1_i4b), x(:n,1_i4b)  )
!
!                   CHECK THAT INVERSE(TRANSFORM(SEQUENCE)) = SEQUENCE.
!
                    x(:n,1_i4b) = x(:n,1_i4b) - x2(:n,1_i4b)
!
                    err = norm( x(:n,1_i4b) )/norm( x2(:n,1_i4b) )
!
                case( 2_i4b, 3_i4b )
!
!                   TESTS 2 and 3: TEST 2D REAL_FFT_FORWARD AND REAL_FFT_BACKWARD.
!
                    dim  = j - 1_i4b
                    dim2 = 3_i4b - dim
!
                    x(:n,:n) = x2(:n,:n)
!
!                   TRANSFORM AND THEN INVERT THE SEQUENCE BACK.
!
                    call real_fft_forward( x(:n,:n), xr(:n,:p), xi(:n,:p), dim  )
                    call real_fft_backward( xr(:n,:p), xi(:n,:p), x(:n,:n), dim  )
!
!                   CHECK THAT INVERSE(TRANSFORM(SEQUENCE)) = SEQUENCE.
!
                    x(:n,:n) = x(:n,:n) - x2(:n,:n)
!
                    err = maxval( norm( x(:n,:n), dim=dim2 )/norm( x2(:n,:n), dim=dim2 ) )
!
            end select
!
!           PRINT THE RESULTS OF THE TESTS.
!
            if ( err<=eps ) then
!
                success = success + 1_i4b
!
            else
!
                write (prtunit,*) 'Test ',m,' is incorrect'
!
                write (prtunit,*) '=> ',err,' > ',eps,' , n= ', n,' , p= ', p,' , case= ', j
!
            end if
!
        end do
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( x, x2, xr, xi )
!
    end do
!
    if ( m/=success ) then
        write (prtunit,*) m - success, 'tests are incorrect'
    end if
!
    write (prtunit,*) success, 'tests are correct'
!
!
! END OF PROGRAM test_real_fft_forward
! ====================================
!
end program test_real_fft_forward
